@@ -1,17 +1,22 @@
 /* apps/enc.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,14 +60,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "err.h"
-#include "envelope.h"
+#include "evp.h"
 #include "objects.h"
 #include "x509.h"
 #ifdef NO_MD5
-#include <md5.h>
-#else
 #include "md5.h"
 #endif
 #include "pem.h"
@@ -80,23 +89,20 @@ int argc;
 char **argv;
 	{
 	char *strbuf=NULL;
-	EVP_ENCODE_CTX basein,baseout;
-	EVP_CIPHER_CTX p;
-	unsigned char *buff=NULL,*buff2=NULL,*btmp,*bufsize=NULL;
+	unsigned char *buff=NULL,*bufsize=NULL;
 	int bsize=BSIZE,verbose=0;
-	int num_in=0,num_out=0;
-	int ret=1,inl,outl,donesome=0;
+	int ret=1,inl;
 	unsigned char key[24],iv[MD5_DIGEST_LENGTH];
 	char *str=NULL;
 	char *hkey=NULL,*hiv=NULL;
-	int enc=1,printkey=0,i,base64=0,done=0;
+	int enc=1,printkey=0,i,base64=0;
+	int debug=0;
 	EVP_CIPHER *cipher=NULL,*c;
 	char *inf=NULL,*outf=NULL;
-	BIO *in=NULL,*out=NULL;
+	BIO *in=NULL,*out=NULL,*b64=NULL,*benc=NULL,*rbio=NULL,*wbio=NULL;
 #define PROG_NAME_SIZE  16
         char pname[PROG_NAME_SIZE];
 
-
 	apps_startup();
 
 	if (bio_err == NULL)
@@ -109,6 +115,11 @@ char **argv;
 		base64=1;
 
 	cipher=EVP_get_cipherbyname(pname);
+	if (!base64 && (cipher == NULL) && (strcmp(pname,"enc") != 0))
+		{
+		BIO_printf(bio_err,"%s is an unknown cipher\n",pname);
+		goto bad;
+		}
 
 	argc--;
 	argv++;
@@ -132,6 +143,8 @@ char **argv;
 			printkey=1;
 		else if	(strcmp(*argv,"-v") == 0)
 			verbose=1;
+		else if	(strcmp(*argv,"-debug") == 0)
+			debug=1;
 		else if	(strcmp(*argv,"-P") == 0)
 			printkey=2;
 		else if	(strcmp(*argv,"-a") == 0)
@@ -151,22 +164,21 @@ char **argv;
 		else if (strcmp(*argv,"-kfile") == 0)
 			{
 			static char buf[128];
-			FILE *in;
+			FILE *infile;
 			char *file;
-			int i;
 
 			if (--argc < 1) goto bad;
 			file= *(++argv);
-			in=fopen(file,"r");
-			if (in == NULL)
+			infile=fopen(file,"r");
+			if (infile == NULL)
 				{
-				fprintf(stderr,"unable to read key from '%s'\n",
+				BIO_printf(bio_err,"unable to read key from '%s'\n",
 					file);
 				goto bad;
 				}
 			buf[0]='\0';
-			fgets(buf,128,in);
-			fclose(in);
+			fgets(buf,128,infile);
+			fclose(infile);
 			i=strlen(buf);
 			if ((i > 0) &&
 				((buf[i-1] == '\n') || (buf[i-1] == '\r')))
@@ -176,7 +188,7 @@ char **argv;
 				buf[--i]='\0';
 			if (i < 1)
 				{
-				fprintf(stderr,"zero length password\n");
+				BIO_printf(bio_err,"zero length password\n");
 				goto bad;
 				}
 			str=buf;
@@ -200,63 +212,73 @@ char **argv;
 			cipher=NULL;
 		else
 			{
-			fprintf(stderr,"unknown option '%s'\n",*argv);
+			BIO_printf(bio_err,"unknown option '%s'\n",*argv);
 bad:
-			fprintf(stderr,"options are\n");
-			fprintf(stderr,"%-14s input file\n","-in <file>");
-			fprintf(stderr,"%-14s output fileencrypt\n","-out <file>");
-			fprintf(stderr,"%-14s encrypt\n","-e");
-			fprintf(stderr,"%-14s decrypt\n","-d");
-			fprintf(stderr,"%-14s base64 encode/decode, depending on encryption flag\n","-a/-base64");
-			fprintf(stderr,"%-14s key is the next argument\n","-k");
-			fprintf(stderr,"%-14s key is the first line of the file argument\n","-kfile");
-			fprintf(stderr,"%-14s key/iv in hex is the next argument\n","-K/-iv");
-			fprintf(stderr,"%-14s print the iv/key (then exit if -P)\n","-[pP]");
-			fprintf(stderr,"%-14s buffer size\n","-bufsize <n>");
+			BIO_printf(bio_err,"options are\n");
+			BIO_printf(bio_err,"%-14s input file\n","-in <file>");
+			BIO_printf(bio_err,"%-14s output fileencrypt\n","-out <file>");
+			BIO_printf(bio_err,"%-14s encrypt\n","-e");
+			BIO_printf(bio_err,"%-14s decrypt\n","-d");
+			BIO_printf(bio_err,"%-14s base64 encode/decode, depending on encryption flag\n","-a/-base64");
+			BIO_printf(bio_err,"%-14s key is the next argument\n","-k");
+			BIO_printf(bio_err,"%-14s key is the first line of the file argument\n","-kfile");
+			BIO_printf(bio_err,"%-14s key/iv in hex is the next argument\n","-K/-iv");
+			BIO_printf(bio_err,"%-14s print the iv/key (then exit if -P)\n","-[pP]");
+			BIO_printf(bio_err,"%-14s buffer size\n","-bufsize <n>");
 
-			fprintf(stderr,"Cipher Types\n");
-			fprintf(stderr,"des     : 56 bit key DES encryption\n");
-			fprintf(stderr,"des_ede :112 bit key ede DES encryption\n");
-			fprintf(stderr,"des_ede3:168 bit key ede DES encryption\n");
+			BIO_printf(bio_err,"Cipher Types\n");
+			BIO_printf(bio_err,"des     : 56 bit key DES encryption\n");
+			BIO_printf(bio_err,"des_ede :112 bit key ede DES encryption\n");
+			BIO_printf(bio_err,"des_ede3:168 bit key ede DES encryption\n");
 #ifndef NO_IDEA
-			fprintf(stderr,"idea    :128 bit key IDEA encryption\n");
+			BIO_printf(bio_err,"idea    :128 bit key IDEA encryption\n");
 #endif
 #ifndef NO_RC4
-			fprintf(stderr,"rc2     :128 bit key RC2 encryption\n");
+			BIO_printf(bio_err,"rc2     :128 bit key RC2 encryption\n");
+#endif
+#ifndef NO_BLOWFISH
+			BIO_printf(bio_err,"bf      :128 bit key BlowFish encryption\n");
 #endif
 #ifndef NO_RC4
-			fprintf(stderr," -%-5s :128 bit key RC4 encryption\n",
+			BIO_printf(bio_err," -%-5s :128 bit key RC4 encryption\n",
 				LN_rc4);
 #endif
 
-			fprintf(stderr," -%-12s -%-12s -%-12s -%-12s",
-				LN_des_ecb,LN_des_cbc,LN_des_cfb,LN_des_ofb);
-			fprintf(stderr," -%-4s (%s)\n",
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
+				LN_des_ecb,LN_des_cbc,
+				LN_des_cfb64,LN_des_ofb64);
+			BIO_printf(bio_err," -%-4s (%s)\n",
 				"des", LN_des_cbc);
 
-			fprintf(stderr," -%-12s -%-12s -%-12s -%-12s",
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
 				LN_des_ede,LN_des_ede_cbc,
-				LN_des_ede_cfb,LN_des_ede_ofb);
-			fprintf(stderr," -desx -none\n");
+				LN_des_ede_cfb64,LN_des_ede_ofb64);
+			BIO_printf(bio_err," -desx -none\n");
 
 
-			fprintf(stderr," -%-12s -%-12s -%-12s -%-12s",
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
 				LN_des_ede3,LN_des_ede3_cbc,
-				LN_des_ede3_cfb,LN_des_ede3_ofb);
-			fprintf(stderr," -%-4s (%s)\n",
+				LN_des_ede3_cfb64,LN_des_ede3_ofb64);
+			BIO_printf(bio_err," -%-4s (%s)\n",
 				"des3", LN_des_ede3_cbc);
 
 #ifndef NO_IDEA
-			fprintf(stderr," -%-12s -%-12s -%-12s -%-12s",
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
 				LN_idea_ecb, LN_idea_cbc,
-				LN_idea_cfb, LN_idea_ofb);
-			fprintf(stderr," -%-4s (%s)\n","idea",LN_idea_cbc);
+				LN_idea_cfb64, LN_idea_ofb64);
+			BIO_printf(bio_err," -%-4s (%s)\n","idea",LN_idea_cbc);
 #endif
 #ifndef NO_RC2
-			fprintf(stderr," -%-12s -%-12s -%-12s -%-12s",
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
 				LN_rc2_ecb, LN_rc2_cbc,
-				LN_rc2_cfb, LN_rc2_ofb);
-			fprintf(stderr," -%-4s (%s)\n","rc2", LN_rc2_cbc);
+				LN_rc2_cfb64, LN_rc2_ofb64);
+			BIO_printf(bio_err," -%-4s (%s)\n","rc2", LN_rc2_cbc);
+#endif
+#ifndef NO_BLOWFISH
+			BIO_printf(bio_err," -%-12s -%-12s -%-12s -%-12s",
+				LN_bf_ecb, LN_bf_cbc,
+				LN_bf_cfb64, LN_bf_ofb64);
+			BIO_printf(bio_err," -%-4s (%s)\n","bf", LN_bf_cbc);
 #endif
 			goto end;
 			}
@@ -266,7 +288,6 @@ bad:
 
 	if (bufsize != NULL)
 		{
-		int i;
 		unsigned long n;
 
 		for (n=0; *bufsize; bufsize++)
@@ -283,7 +304,7 @@ bad:
 			}
 		if (*bufsize != '\0')
 			{
-			fprintf(stderr,"invalid 'bufsize' specified.\n");
+			BIO_printf(bio_err,"invalid 'bufsize' specified.\n");
 			goto end;
 			}
 
@@ -291,15 +312,14 @@ bad:
 		if (n < 80) n=80;
 
 		bsize=(int)n;
-		if (verbose) fprintf(stderr,"bufsize=%d\n",bsize);
+		if (verbose) BIO_printf(bio_err,"bufsize=%d\n",bsize);
 		}
 
-	strbuf=malloc(SIZE);
-	buff=(unsigned char *)malloc(EVP_ENCODE_LENGTH(bsize));
-	buff2=(unsigned char *)malloc(EVP_ENCODE_LENGTH(bsize));
-	if ((buff == NULL) || (buff2 == NULL) || (strbuf == NULL))
+	strbuf=Malloc(SIZE);
+	buff=(unsigned char *)Malloc(EVP_ENCODE_LENGTH(bsize));
+	if ((buff == NULL) || (strbuf == NULL))
 		{
-		fprintf(stderr,"malloc failure\n");
+		BIO_printf(bio_err,"Malloc failure %ld\n",(long)EVP_ENCODE_LENGTH(bsize));
 		goto end;
 		}
 
@@ -310,6 +330,13 @@ bad:
 		ERR_print_errors(bio_err);
 		goto end;
 		}
+	if (debug)
+		{
+		BIO_set_callback(in,BIO_debug_callback);
+		BIO_set_callback(out,BIO_debug_callback);
+		BIO_set_callback_arg(in,bio_err);
+		BIO_set_callback_arg(out,bio_err);
+		}
 
 	if (inf == NULL)
 		BIO_set_fp(in,stdin,BIO_NOCLOSE);
@@ -329,7 +356,7 @@ bad:
 			char buf[200];
 
 			sprintf(buf,"enter %s %s password:",
-				OBJ_nid2ln(cipher->type),
+				OBJ_nid2ln(EVP_CIPHER_nid(cipher)),
 				(enc)?"encryption":"decryption");
 			strbuf[0]='\0';
 			i=EVP_read_pw_string((char *)strbuf,SIZE,buf,enc);
@@ -345,7 +372,7 @@ bad:
 				}
 			if (i < 0)
 				{
-				fprintf(stderr,"bad password read\n");
+				BIO_printf(bio_err,"bad password read\n");
 				goto end;
 				}
 			}
@@ -369,15 +396,23 @@ bad:
 			}
 		if ((hiv != NULL) && !set_hex(hiv,iv,8))
 			{
-			fprintf(stderr,"invalid hex iv value\n");
+			BIO_printf(bio_err,"invalid hex iv value\n");
 			goto end;
 			}
 		if ((hkey != NULL) && !set_hex(hkey,key,24))
 			{
-			fprintf(stderr,"invalid hex key value\n");
+			BIO_printf(bio_err,"invalid hex key value\n");
 			goto end;
 			}
-		EVP_CipherInit(&p,cipher,key,iv,enc);
+
+		if ((benc=BIO_new(BIO_f_cipher())) == NULL)
+			goto end;
+		BIO_set_cipher(benc,cipher,key,iv,enc);
+		if (debug)
+			{
+			BIO_set_callback(benc,BIO_debug_callback);
+			BIO_set_callback_arg(benc,bio_err);
+			}
 
 		if (printkey)
 			{
@@ -415,124 +450,57 @@ bad:
 			}
 		}
 
+	rbio=in;
+	wbio=out;
+
 	if (base64)
 		{
+		if ((b64=BIO_new(BIO_f_base64())) == NULL)
+			goto end;
+		if (debug)
+			{
+			BIO_set_callback(b64,BIO_debug_callback);
+			BIO_set_callback_arg(b64,bio_err);
+			}
 		if (enc)
-			EVP_EncodeInit(&baseout);
+			wbio=BIO_push(b64,wbio);
 		else
-			EVP_DecodeInit(&basein);
+			rbio=BIO_push(b64,rbio);
 		}
 
+	/* Only encrypt/decrypt as we write the file */
+	if (benc != NULL)
+		wbio=BIO_push(benc,wbio);
+
 	for (;;)
 		{
-		/* While we have not started base64 decoding,
-		 * read a line at a time until we hit a valid line */
-		if (base64 && !enc && !donesome)
+		inl=BIO_read(rbio,(char *)buff,bsize);
+		if (inl <= 0) break;
+		if (BIO_write(wbio,(char *)buff,inl) != inl)
 			{
-			buff[0]='\0';
-			inl=BIO_gets(in,(char *)buff,bsize);
-			if (buff[0] == '\0') break;
-			}
-		else
-			inl=BIO_read(in,(char *)buff,bsize);
-
-		if (inl == 0) break;
-		num_in+=inl;
-		outl=inl;
-
-		if (base64 && !enc)
-			{
-			i=EVP_DecodeUpdate(&basein,buff2,&outl,buff,outl);
-			if ((i <= 0) && !donesome)
-				{
-				EVP_DecodeInit(&basein);
-				continue;
-				}
-			donesome=1;
-			if (i <= 0) done=1;
-			btmp=buff2; buff2=buff; buff=btmp;
-			}
-
-		if (cipher != NULL)
-			{
-			EVP_CipherUpdate(&p,buff2,&outl,buff,outl);
-			btmp=buff2; buff2=buff; buff=btmp;
-			}
-
-		if (base64 && enc)
-			{
-			EVP_EncodeUpdate(&baseout,buff2,&outl,buff,outl);
-			btmp=buff2; buff2=buff; buff=btmp;
-			}
-
-		num_out+=outl;
-		if (outl != 0)
-			BIO_write(out,(char *)buff,outl);
-		outl=0;
-		if (done) break;
-		}
-
-	/* Finish off any pending base64 input */
-	if (base64 && !enc)
-		{
-		EVP_DecodeFinal(&basein,buff2,&outl);
-		btmp=buff2; buff2=buff; buff=btmp;
-
-		if ((cipher != NULL) && (outl != 0))
-			{
-			EVP_CipherUpdate(&p,buff2,&outl,buff,outl);
-			btmp=buff2; buff2=buff; buff=btmp;
-			}
-		num_out+=outl;
-		if (outl != 0)
-			BIO_write(out,(char *)buff,outl);
-		outl=0;
-		}
-
-	/* Finish off any pending cipher input */
-	if (cipher != NULL)
-		{
-		if (!EVP_CipherFinal(&p,buff,&outl))
-			{
-			fprintf(stderr,"error in EVP_CipherFinal\n");
+			BIO_printf(bio_err,"error writing output file\n");
 			goto end;
 			}
-		memset(&p,0,sizeof(p));
-		if (base64 && enc)
-			{
-			EVP_EncodeUpdate(&baseout,buff2,&outl,buff,outl);
-			btmp=buff2; buff2=buff; buff=btmp;
-			num_out+=outl;
-			if (outl != 0)
-				BIO_write(out,(char *)buff,outl);
-			outl=0;
-			}
 		}
-
-	/* Finish off any pending base64 output */
-	if (base64 && enc)
+	if (!BIO_flush(wbio))
 		{
-		EVP_EncodeFinal(&baseout,buff2,&outl);
-		btmp=buff2; buff2=buff; buff=btmp;
+		BIO_printf(bio_err,"bad decrypt\n");
+		goto end;
 		}
 
-	/* Finish off any pending output */
-	num_out+=outl;
-	if (outl != 0)
-		BIO_write(out,(char *)buff,outl);
-
 	ret=0;
 	if (verbose)
 		{
-		fprintf(stderr,"bytes read   :%8d\n",num_in);
-		fprintf(stderr,"bytes written:%8d\n",num_out);
+		BIO_printf(bio_err,"bytes read   :%8ld\n",BIO_number_read(in));
+		BIO_printf(bio_err,"bytes written:%8ld\n",BIO_number_written(out));
 		}
 end:
-	if (strbuf != NULL) free(strbuf);
-	if (buff != NULL) free(buff);
-	if (buff2 != NULL) free(buff2);
+	if (strbuf != NULL) Free(strbuf);
+	if (buff != NULL) Free(buff);
 	if (in != NULL) BIO_free(in);
 	if (out != NULL) BIO_free(out);
+	if (benc != NULL) BIO_free(benc);
+	if (b64 != NULL) BIO_free(b64);
 	EXIT(ret);
 	}
 
@@ -547,7 +515,7 @@ int size;
 	n=strlen(in);
 	if (n > (size*2))
 		{
-		fprintf(stderr,"hex string is too long\n");
+		BIO_printf(bio_err,"hex string is too long\n");
 		return(0);
 		}
 	memset(out,0,size);
@@ -564,7 +532,7 @@ int size;
 			j=j-'a'+10;
 		else
 			{
-			fprintf(stderr,"non-hex digit\n");
+			BIO_printf(bio_err,"non-hex digit\n");
 			return(0);
 			}
 		if (i&1)
