@@ -1,17 +1,22 @@
 /* apps/pkcs7.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -52,7 +63,7 @@
 #include "apps.h"
 #include "err.h"
 #include "objects.h"
-#include "envelope.h"
+#include "evp.h"
 #include "x509.h"
 #include "pkcs7.h"
 #include "pem.h"
@@ -78,7 +89,7 @@ char **argv;
 	EVP_CIPHER *enc=NULL;
 	BIO *in=NULL,*out=NULL;
 	int informat,outformat;
-	char *infile,*outfile,*prog;
+	char *infile,*outfile,*prog,buf[256];
 	int print_certs=0;
 	int ret=0;
 
@@ -120,17 +131,19 @@ char **argv;
 			}
 		else if (strcmp(*argv,"-print_certs") == 0)
 			print_certs=1;
+#ifndef NO_DES
 		else if (strcmp(*argv,"-des") == 0)
 			enc=EVP_des_cbc();
 		else if (strcmp(*argv,"-des3") == 0)
 			enc=EVP_des_ede3_cbc();
+#endif
 #ifndef NO_IDEA
 		else if (strcmp(*argv,"-idea") == 0)
 			enc=EVP_idea_cbc();
 #endif
 		else
 			{
-			fprintf(stderr,"unknown option %s\n",*argv);
+			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
 			break;
 			}
@@ -141,17 +154,17 @@ char **argv;
 	if (badops)
 		{
 bad:
-		fprintf(stderr,"%s [options] <infile >outfile\n",prog);
-		fprintf(stderr,"where options are\n");
-		fprintf(stderr," -inform arg   input format - one of DER TXT PEM\n");
-		fprintf(stderr," -outform arg  output format - one of DER TXT PEM\n");
-		fprintf(stderr," -in arg       inout file\n");
-		fprintf(stderr," -out arg      output file\n");
-		fprintf(stderr," -print_certs  print any certs or crl in the input\n");
-		fprintf(stderr," -des          encrypt PEM output with cbc des\n");
-		fprintf(stderr," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
+		BIO_printf(bio_err,"%s [options] <infile >outfile\n",prog);
+		BIO_printf(bio_err,"where options are\n");
+		BIO_printf(bio_err," -inform arg   input format - one of DER TXT PEM\n");
+		BIO_printf(bio_err," -outform arg  output format - one of DER TXT PEM\n");
+		BIO_printf(bio_err," -in arg       inout file\n");
+		BIO_printf(bio_err," -out arg      output file\n");
+		BIO_printf(bio_err," -print_certs  print any certs or crl in the input\n");
+		BIO_printf(bio_err," -des          encrypt PEM output with cbc des\n");
+		BIO_printf(bio_err," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
 #ifndef NO_IDEA
-		fprintf(stderr," -idea         encrypt PEM output with cbc idea\n");
+		BIO_printf(bio_err," -idea         encrypt PEM output with cbc idea\n");
 #endif
 		EXIT(1);
 		}
@@ -184,12 +197,12 @@ bad:
 		p7=PEM_read_bio_PKCS7(in,NULL,NULL);
 	else
 		{
-		fprintf(stderr,"bad input format specified for pkcs7 object\n");
+		BIO_printf(bio_err,"bad input format specified for pkcs7 object\n");
 		goto end;
 		}
 	if (p7 == NULL)
 		{
-		fprintf(stderr,"unable to load PKCS7 object\n");
+		BIO_printf(bio_err,"unable to load PKCS7 object\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
@@ -207,7 +220,6 @@ bad:
 
 	if (print_certs)
 		{
-		int i;
 		STACK *certs=NULL;
 		STACK *crls=NULL;
 
@@ -228,23 +240,22 @@ bad:
 
 		if (certs != NULL)
 			{
-			char *a;
 			X509 *x;
 
 			for (i=0; i<sk_num(certs); i++)
 				{
 				x=(X509 *)sk_value(certs,i);
 
-				a=X509_NAME_oneline(X509_get_subject_name(x));
+				X509_NAME_oneline(X509_get_subject_name(x),
+					buf,256);
 				BIO_puts(out,"subject=");
-				BIO_puts(out,a);
-				Free(a);
+				BIO_puts(out,buf);
 
-				a=X509_NAME_oneline(X509_get_issuer_name(x));
+				X509_NAME_oneline(X509_get_issuer_name(x),
+					buf,256);
 				BIO_puts(out,"\nissuer= ");
-				BIO_puts(out,a);
+				BIO_puts(out,buf);
 				BIO_puts(out,"\n");
-				Free(a);
 
 				PEM_write_bio_X509(out,x);
 				BIO_puts(out,"\n");
@@ -253,16 +264,14 @@ bad:
 		if (crls != NULL)
 			{
 			X509_CRL *crl;
-			char *a;
 
 			for (i=0; i<sk_num(crls); i++)
 				{
 				crl=(X509_CRL *)sk_value(crls,i);
 
-				a=X509_NAME_oneline(crl->crl->issuer);
+				X509_NAME_oneline(crl->crl->issuer,buf,256);
 				BIO_puts(out,"issuer= ");
-				BIO_puts(out,a);
-				Free(a);
+				BIO_puts(out,buf);
 
 				BIO_puts(out,"\nlast update=");
 				ASN1_UTCTIME_print(out,crl->crl->lastUpdate);
@@ -284,13 +293,13 @@ bad:
 	else if (outformat == FORMAT_PEM)
 		i=PEM_write_bio_PKCS7(out,p7);
 	else	{
-		fprintf(stderr,"bad output format specified for outfile\n");
+		BIO_printf(bio_err,"bad output format specified for outfile\n");
 		goto end;
 		}
 
 	if (!i)
 		{
-		fprintf(stderr,"unable to write pkcs7 object\n");
+		BIO_printf(bio_err,"unable to write pkcs7 object\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
