@@ -1,17 +1,22 @@
 /* apps/rsa.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -50,10 +61,10 @@
 #include <string.h>
 #include <time.h>
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "err.h"
 #include "rsa.h"
-#include "envelope.h"
+#include "evp.h"
 #include "x509.h"
 #include "pem.h"
 
@@ -126,17 +137,9 @@ char **argv;
 			text=1;
 		else if (strcmp(*argv,"-modulus") == 0)
 			modulus=1;
-		else if (strcmp(*argv,"-des") == 0)
-			enc=EVP_des_cbc();
-		else if (strcmp(*argv,"-des3") == 0)
-			enc=EVP_des_ede3_cbc();
-#ifndef NO_IDEA
-		else if (strcmp(*argv,"-idea") == 0)
-			enc=EVP_idea_cbc();
-#endif
-		else
+		else if ((enc=EVP_get_cipherbyname(&(argv[0][1]))) == NULL)
 			{
-			fprintf(stderr,"unknown option %s\n",*argv);
+			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
 			break;
 			}
@@ -147,20 +150,20 @@ char **argv;
 	if (badops)
 		{
 bad:
-		fprintf(stderr,"%s [options] <infile >outfile\n",prog);
-		fprintf(stderr,"where options are\n");
-		fprintf(stderr," -inform arg   input format - one of DER NET PEM\n");
-		fprintf(stderr," -outform arg  output format - one of DER NET PEM\n");
-		fprintf(stderr," -in arg       inout file\n");
-		fprintf(stderr," -out arg      output file\n");
-		fprintf(stderr," -des          encrypt PEM output with cbc des\n");
-		fprintf(stderr," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
+		BIO_printf(bio_err,"%s [options] <infile >outfile\n",prog);
+		BIO_printf(bio_err,"where options are\n");
+		BIO_printf(bio_err," -inform arg   input format - one of DER NET PEM\n");
+		BIO_printf(bio_err," -outform arg  output format - one of DER NET PEM\n");
+		BIO_printf(bio_err," -in arg       inout file\n");
+		BIO_printf(bio_err," -out arg      output file\n");
+		BIO_printf(bio_err," -des          encrypt PEM output with cbc des\n");
+		BIO_printf(bio_err," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
 #ifndef NO_IDEA
-		fprintf(stderr," -idea         encrypt PEM output with cbc idea\n");
+		BIO_printf(bio_err," -idea         encrypt PEM output with cbc idea\n");
 #endif
-		fprintf(stderr," -text         print the key in text\n");
-		fprintf(stderr," -noout        don't print key out\n");
-		fprintf(stderr," -modulus      print the RSA key modulus\n");
+		BIO_printf(bio_err," -text         print the key in text\n");
+		BIO_printf(bio_err," -noout        don't print key out\n");
+		BIO_printf(bio_err," -modulus      print the RSA key modulus\n");
 		goto end;
 		}
 
@@ -185,7 +188,7 @@ bad:
 			}
 		}
 
-	fprintf(stderr,"read RSA private key\n");
+	BIO_printf(bio_err,"read RSA private key\n");
 	if	(informat == FORMAT_ASN1)
 		rsa=d2i_RSAPrivateKey_bio(in,NULL);
 #ifndef NO_RC4
@@ -193,7 +196,7 @@ bad:
 		{
 		BUF_MEM *buf=NULL;
 		unsigned char *p;
-		int size=0,i;
+		int size=0;
 
 		buf=BUF_MEM_new();
 		for (;;)
@@ -219,12 +222,12 @@ bad:
 		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
 	else
 		{
-		fprintf(stderr,"bad input format specified for key\n");
+		BIO_printf(bio_err,"bad input format specified for key\n");
 		goto end;
 		}
 	if (rsa == NULL)
 		{
-		fprintf(stderr,"unable to load Private Key\n");
+		BIO_printf(bio_err,"unable to load Private Key\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
@@ -256,7 +259,7 @@ bad:
 		}
 
 	if (noout) goto end;
-	fprintf(stderr,"writing RSA private key\n");
+	BIO_printf(bio_err,"writing RSA private key\n");
 	if 	(outformat == FORMAT_ASN1)
 		i=i2d_RSAPrivateKey_bio(out,rsa);
 #ifndef NO_RC4
@@ -269,7 +272,7 @@ bad:
 		size=i2d_Netscape_RSA(rsa,NULL,NULL);
 		if ((p=(unsigned char *)Malloc(size)) == NULL)
 			{
-			fprintf(stderr,"Malloc failure\n");
+			BIO_printf(bio_err,"Malloc failure\n");
 			goto end;
 			}
 		pp=p;
@@ -281,12 +284,12 @@ bad:
 	else if (outformat == FORMAT_PEM)
 		i=PEM_write_bio_RSAPrivateKey(out,rsa,enc,NULL,0,NULL);
 	else	{
-		fprintf(stderr,"bad output format specified for outfile\n");
+		BIO_printf(bio_err,"bad output format specified for outfile\n");
 		goto end;
 		}
 	if (!i)
 		{
-		fprintf(stderr,"unable to write private key\n");
+		BIO_printf(bio_err,"unable to write private key\n");
 		ERR_print_errors(bio_err);
 		}
 	else
