@@ -1,17 +1,22 @@
 /* apps/s_time.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -45,6 +56,8 @@
  * [including the GNU Public Licence.]
  */
 
+#undef NO_SHUTDOWN
+
 /*-----------------------------------------
    cntime - SSL client connection timer program
    Written and donated by Larry Streepy <streepy@healthcare.com>
@@ -119,7 +132,7 @@ struct tms {
 
 #define ioctl ioctlsocket
 
-#define SSL_HOST_NAME	"localhost"
+#define SSL_CONNECT_NAME	"localhost:4433"
 
 /*#define TEST_CERT "client.pem" */ /* no default cert. */
 
@@ -135,11 +148,11 @@ extern int verify_depth;
 extern int verify_error;
 
 #ifndef NOPROTO
-static void usage(void);
+static void s_time_usage(void);
 static int parseArgs( int argc, char **argv );
 static SSL *doConnection( SSL *scon );
 #else
-static void usage();
+static void s_time_usage();
 static int parseArgs();
 static SSL *doConnection();
 #endif
@@ -149,19 +162,24 @@ static SSL *doConnection();
  * Static data declarations
  */
 
-static int port=PORT;
-static char *host=SSL_HOST_NAME;
+/* static char *port=PORT_STR;*/
+static char *host=SSL_CONNECT_NAME;
 static char *t_cert_file=NULL;
 static char *t_key_file=NULL;
 static char *CApath=NULL;
 static char *CAfile=NULL;
-static char *cipher=NULL;
-static int verify = SSL_VERIFY_NONE;
+static char *tm_cipher=NULL;
+static int tm_verify = SSL_VERIFY_NONE;
 static int maxTime = SECONDS;
-static SSL_CTX *ctx=NULL;
+static SSL_CTX *tm_ctx=NULL;
+static SSL_METHOD *s_time_meth=NULL;
+static char *s_www_path=NULL;
+static long bytes_read=0; 
+static int st_bugs=0;
+static int perform=0;
 
 #ifdef FIONBIO
-static int nbio=0;
+static int t_nbio=0;
 #endif
 
 #ifdef WIN32
@@ -171,7 +189,7 @@ static int exitNow = 0;		/* Set when it's time to exit main */
 /***********************************************************************
  * usage - display usage message
  */
-static void usage()
+static void s_time_usage()
 {
 	static char umsg[] = "\
 -time arg     - max number of seconds to collect data, default %d\n\
@@ -181,7 +199,7 @@ static void usage()
                 not specified but cert fill is.\n\
 -CApath arg   - PEM format directory of CA's\n\
 -CAfile arg   - PEM format file of CA's\n\
--cipher       - prefered cipher to use, ':' seperated list of the following\n\
+-cipher       - prefered cipher to use, ':' separated list of the following\n\
                 NULL-MD5     RC4-MD5      EXP-RC4-MD5 \n\
                 IDEA-CBC-MD5 RC2-CBC-MD5  EXP-RC2-CBC-MD5\n\
                 DES-CBC-MD5  DES-CBC-SHA  DES-CBC3-MD5\n\
@@ -189,9 +207,11 @@ static void usage()
 
 	printf( "usage: client <args>\n\n" );
 
-	printf("-host arg     - host or IP to connect to (default is %s)\n",
-			SSL_HOST_NAME);
+/*	printf("-host arg     - host or IP to connect to (default is %s)\n",
+			SSL_CONNECT_NAME);
 	printf("-port arg     - port to connect to (default is %d)\n",PORT);
+*/
+	printf("-connect host:port - host:port to connect to (default is %s)\n",SSL_CONNECT_NAME);
 #ifdef FIONBIO
 	printf("-nbio         - Run with non-blocking IO\n");
 #endif
@@ -210,38 +230,52 @@ char **argv;
     int badop = 0;
 
     verify_depth=0;
-    verify_error=VERIFY_OK;
+    verify_error=X509_V_OK;
 #ifdef FIONBIO
-    nbio=0;
+    t_nbio=0;
 #endif
 
 	apps_startup();
 
 	if (bio_err == NULL)
-		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
-			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+		bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
 
     argc--;
     argv++;
 
     while (argc >= 1) {
-	if( strcmp(*argv,"-host") == 0) {
+	if (strcmp(*argv,"-connect") == 0)
+		{
+		if (--argc < 1) goto bad;
+		host= *(++argv);
+		}
+#if 0
+	else if( strcmp(*argv,"-host") == 0)
+		{
+		if (--argc < 1) goto bad;
+		host= *(++argv);
+		}
+	else if( strcmp(*argv,"-port") == 0)
+		{
+		if (--argc < 1) goto bad;
+		port= *(++argv);
+		}
+#endif
+	else if (strcmp(*argv,"-reuse") == 0)
+		perform=2;
+	else if (strcmp(*argv,"-new") == 0)
+		perform=1;
+	else if (strcmp(*argv,"-connect") == 0)
+		{
+		if (--argc < 1) goto bad;
+		host= *(++argv);
+		}
+	else if( strcmp(*argv,"-verify") == 0) {
 
-	    if (--argc < 1) goto bad;
-	    host= *(++argv);
-
-	} else if( strcmp(*argv,"-port") == 0) {
-
-	    if (--argc < 1) goto bad;
-	    port=atoi(*(++argv));
-	    if (port == 0) goto bad;
-
-	} else if( strcmp(*argv,"-verify") == 0) {
-
-	    verify=SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE;
+	    tm_verify=SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE;
 	    if (--argc < 1) goto bad;
 	    verify_depth=atoi(*(++argv));
-	    fprintf(stderr,"verify depth is %d\n",verify_depth);
+	    BIO_printf(bio_err,"verify depth is %d\n",verify_depth);
 
 	} else if( strcmp(*argv,"-cert") == 0) {
 
@@ -266,12 +300,27 @@ char **argv;
 	} else if( strcmp(*argv,"-cipher") == 0) {
 
 	    if (--argc < 1) goto bad;
-	    cipher= *(++argv);
+	    tm_cipher= *(++argv);
 	}
 #ifdef FIONBIO
 	else if(strcmp(*argv,"-nbio") == 0) {
-	    nbio=1;
+	    t_nbio=1;
 	}
+#endif
+	else if(strcmp(*argv,"-www") == 0)
+		{
+		if (--argc < 1) goto bad;
+		s_www_path= *(++argv);
+		}
+	else if(strcmp(*argv,"-bugs") == 0)
+	    st_bugs=1;
+#ifndef NO_SSL2
+	else if(strcmp(*argv,"-ssl2") == 0)
+	    s_time_meth=SSLv2_client_method();
+#endif
+#ifndef NO_SSL3
+	else if(strcmp(*argv,"-ssl3") == 0)
+	    s_time_meth=SSLv3_client_method();
 #endif
 	else if( strcmp(*argv,"-time") == 0) {
 
@@ -279,7 +328,7 @@ char **argv;
 	    maxTime= atoi(*(++argv));
 	}
 	else {
-	    fprintf(stderr,"unknown option %s\n",*argv);
+	    BIO_printf(bio_err,"unknown option %s\n",*argv);
 	    badop=1;
 	    break;
 	}
@@ -288,9 +337,11 @@ char **argv;
 	argv++;
     }
 
+    if (perform == 0) perform=3;
+
     if(badop) {
 bad:
-		usage();
+		s_time_usage();
 		return -1;
     }
 
@@ -303,10 +354,10 @@ bad:
 #define START	0
 #define STOP	1
 
-static double Time_F(s)
+static double tm_Time_F(s)
 int s;
 	{
-	double ret;
+	static double ret;
 #ifdef TIMES
 	static struct tms tstart,tend;
 
@@ -345,39 +396,57 @@ char **argv;
 	{
 	double totalTime = 0.0;
 	int nConn = 0;
-	SSL *scon, *savecon=NULL;
+	SSL *scon=NULL;
 	long finishtime=0;
-	int ret=1;
+	int ret=1,i;
+	MS_STATIC char buf[1024*8];
 
-	if ((ctx=SSL_CTX_new()) == NULL) return(1);
+#if !defined(NO_SSL2) && !defined(NO_SSL3)
+	s_time_meth=SSLv23_client_method();
+#elif !defined(NO_SSL3)
+	s_time_meth=SSLv3_client_method();
+#elif !defined(NO_SSL2)
+	s_time_meth=SSLv2_client_method();
+#endif
 
 	/* parse the command line arguments */
 	if( parseArgs( argc, argv ) < 0 )
 		goto end;
 
+	SSLeay_add_ssl_algorithms();
+	if ((tm_ctx=SSL_CTX_new(s_time_meth)) == NULL) return(1);
+
+	if (st_bugs) SSL_CTX_set_options(tm_ctx,SSL_OP_ALL);
+	SSL_CTX_set_cipher_list(tm_ctx,tm_cipher);
+	if(!set_cert_stuff(tm_ctx,t_cert_file,t_key_file)) 
+		goto end;
+
 	SSL_load_error_strings();
 
-	if ((!SSL_load_verify_locations(ctx,CAfile,CApath)) ||
-		(!SSL_set_default_verify_paths(ctx)))
+	if ((!SSL_CTX_load_verify_locations(tm_ctx,CAfile,CApath)) ||
+		(!SSL_CTX_set_default_verify_paths(tm_ctx)))
 		{
-		fprintf(stderr,"error seting default verify locations\n");
+		BIO_printf(bio_err,"error seting default verify locations\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
 
-	if (cipher == NULL)
-		cipher = getenv("SSL_CIPHER");
+	if (tm_cipher == NULL)
+		tm_cipher = getenv("SSL_CIPHER");
 
-	if( cipher == NULL ) {
+	if (tm_cipher == NULL ) {
 		fprintf( stderr, "No CIPHER specified\n" );
 /*		EXIT(1); */
 	}
 
+	if (!(perform & 1)) goto next;
 	printf( "Collecting connection statistics for %d seconds\n", maxTime );
 
 	/* Loop and time how long it takes to make connections */
 
-	finishtime=(long)time(NULL)+SECONDS;
+	bytes_read=0;
+	finishtime=(long)time(NULL)+maxTime;
+	tm_Time_F(START);
 	for (;;)
 		{
 		if (finishtime < time(NULL)) break;
@@ -390,39 +459,74 @@ char **argv;
 			goto end;
 #endif
 
-		Time_F(START);
-		
 		if( (scon = doConnection( NULL )) == NULL )
 			goto end;
 
+		if (s_www_path != NULL)
+			{
+			sprintf(buf,"GET %s HTTP/1.0\r\n\r\n",s_www_path);
+			SSL_write(scon,buf,strlen(buf));
+			while ((i=SSL_read(scon,buf,sizeof(buf))) > 0)
+				bytes_read+=i;
+			}
+
+#ifdef NO_SHUTDOWN
+		SSL_set_shutdown(scon,SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+#else
+		SSL_shutdown(scon);
+#endif
 		SHUTDOWN(SSL_get_fd(scon));
-		SSL_free( scon );
 
-		totalTime += Time_F(STOP); /* Add the time for this iteration */
 		nConn += 1;
-		fputc( '#', stdout ); fflush(stdout);
-	}
+		fputc(SSL_session_reused(scon)?'r':
+			(SSL_version(scon))+'0', stdout );
+		fflush(stdout);
 
-	printf( "\n\n%d connections in %.2fs; %.2f connections/user sec\n",
-			nConn, totalTime, ((double)nConn/totalTime) );
-	printf( "%d connections in %ld real seconds\n",nConn,
-		time(NULL)-finishtime+SECONDS);
+		SSL_free( scon );
+		scon=NULL;
+		}
+	totalTime += tm_Time_F(STOP); /* Add the time for this iteration */
+
+	i=(int)(time(NULL)-finishtime+maxTime);
+	printf( "\n\n%d connections in %.2fs; %.2f connections/user sec, bytes read %ld\n", nConn, totalTime, ((double)nConn/totalTime),bytes_read);
+	printf( "%d connections in %ld real seconds, %ld bytes read per connection\n",nConn,time(NULL)-finishtime+maxTime,bytes_read/nConn);
 
 	/* Now loop and time connections using the same session id over and over */
 
+next:
+	if (!(perform & 2)) goto end;
 	printf( "\n\nNow timing with session id reuse.\n" );
 
 	/* Get an SSL object so we can reuse the session id */
-	if( (savecon = doConnection( NULL )) == NULL )
+	if( (scon = doConnection( NULL )) == NULL )
 		{
 		fprintf( stderr, "Unable to get connection\n" );
 		goto end;
 		}
+
+	if (s_www_path != NULL)
+		{
+		sprintf(buf,"GET %s HTTP/1.0\r\n\r\n",s_www_path);
+		SSL_write(scon,buf,strlen(buf));
+		while (SSL_read(scon,buf,sizeof(buf)) > 0)
+			;
+		}
+#ifdef NO_SHUTDOWN
+	SSL_set_shutdown(scon,SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+#else
+	SSL_shutdown(scon);
+#endif
+	SHUTDOWN(SSL_get_fd(scon));
+
 	nConn = 0;
 	totalTime = 0.0;
 
-	finishtime=time(NULL)+SECONDS;
+	finishtime=time(NULL)+maxTime;
 
+	printf( "starting\n" );
+	bytes_read=0;
+	tm_Time_F(START);
+		
 	for (;;)
 		{
 		if (finishtime < time(NULL)) break;
@@ -435,35 +539,43 @@ char **argv;
 			goto end;
 #endif
 
-		Time_F(START);
-		
-	 	if( (scon = doConnection( savecon )) == NULL )
+	 	if( (doConnection( scon )) == NULL )
 			goto end;
 
+		if (s_www_path)
+			{
+			sprintf(buf,"GET %s HTTP/1.0\r\n\r\n",s_www_path);
+			SSL_write(scon,buf,strlen(buf));
+			while ((i=SSL_read(scon,buf,sizeof(buf))) > 0)
+				bytes_read+=i;
+			}
+
+#ifdef NO_SHUTDOWN
+		SSL_set_shutdown(scon,SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+#else
+		SSL_shutdown(scon);
+#endif
 		SHUTDOWN(SSL_get_fd(scon));
-		SSL_free( scon );
-		scon=NULL;
-
 	
-		totalTime += Time_F(STOP); /* Add the time for this iteration*/
 		nConn += 1;
-		fputc( '#', stdout ); fflush(stdout);
+		fputc(SSL_session_reused(scon)?'r':
+			(SSL_version(scon))+'0', stdout );
+		fflush(stdout);
 		}
+	totalTime += tm_Time_F(STOP); /* Add the time for this iteration*/
 
 
-	printf( "\n\n%d connections in %.2fs; %.2f connections/sec\n",
-			nConn, totalTime, ((double)nConn/totalTime) );
-	printf( "%d connections in %ld real seconds\n",nConn,
-		time(NULL)-finishtime+SECONDS);
+	printf( "\n\n%d connections in %.2fs; %.2f connections/user sec, bytes read %ld\n", nConn, totalTime, ((double)nConn/totalTime),bytes_read);
+	printf( "%d connections in %ld real seconds, %ld bytes read per connection\n",nConn,time(NULL)-finishtime+maxTime,bytes_read/nConn);
 
 	ret=0;
 end:
-	if (savecon != NULL) SSL_free(savecon);
+	if (scon != NULL) SSL_free(scon);
 
-	if (ctx != NULL)
+	if (tm_ctx != NULL)
 		{
-		SSL_CTX_free(ctx);
-		ctx=NULL;
+		SSL_CTX_free(tm_ctx);
+		tm_ctx=NULL;
 		}
 	EXIT(ret);
 	}
@@ -479,60 +591,60 @@ static SSL *
 doConnection(scon)
 SSL *scon;
 	{
+	BIO *conn;
 	SSL *serverCon;
-	int s;
 	int width, i;
 	fd_set readfds;
 
-	if(init_client(&s,host,port) == 0) {
-		perror( "couldn't connect" );
-		/* SHUTDOWN(s); */
-		return NULL;
-	}
+	if ((conn=BIO_new(BIO_s_connect())) == NULL)
+		return(NULL);
 
-	serverCon=(SSL *)SSL_new(ctx);
-	SSL_set_fd(serverCon,s);
+/*	BIO_set_port(conn,port);*/
+	BIO_set_hostname(conn,host);
 
+	if (scon == NULL)
+		serverCon=(SSL *)SSL_new(tm_ctx);
+	else
+		{
+		serverCon=scon;
+		SSL_set_connect_state(serverCon);
+		}
+
+	SSL_set_bio(serverCon,conn,conn);
+
+#if 0
 	if( scon != NULL )
 		SSL_set_session(serverCon,SSL_get_session(scon));
-
-/*	SSL_set_verify(serverCon,verify,verify_callback);*/
-	SSL_CTX_set_cipher_list(ctx,cipher);
-
-	if(!set_cert_stuff(ctx,t_cert_file,t_key_file)) {
-		SHUTDOWN(SSL_get_fd(serverCon));
-		return NULL;
-	}
+#endif
 
 	/* ok, lets connect */
-	do	{
-		width=SSL_get_fd(serverCon)+1;
-		for(;;) {
-			i=SSL_connect(serverCon);
-			if (should_retry(i))
-				{
-				fprintf(stderr,"DELAY\n");
-
-				FD_ZERO(&readfds);
-				FD_SET(SSL_get_fd(serverCon),&readfds);
-				select(width,&readfds,NULL,NULL,NULL);
-				continue;
-				}
-			break;
-			}
-		if(i <= 0)
+	for(;;) {
+		i=SSL_connect(serverCon);
+		if (BIO_sock_should_retry(i))
 			{
-			fprintf(stderr,"ERROR\n");
-			if (verify_error != VERIFY_OK)
-				fprintf(stderr,"verify error:%s\n",
-						X509_cert_verify_error_string(verify_error));
-			else
-				ERR_print_errors(bio_err);
-			return NULL;
+			BIO_printf(bio_err,"DELAY\n");
+
+			i=SSL_get_fd(serverCon);
+			width=i+1;
+			FD_ZERO(&readfds);
+			FD_SET(i,&readfds);
+			select(width,&readfds,NULL,NULL,NULL);
+			continue;
 			}
-		} while (0);
-													
-	SHUTDOWN(SSL_get_fd(serverCon));
+		break;
+		}
+	if(i <= 0)
+		{
+		BIO_printf(bio_err,"ERROR\n");
+		if (verify_error != X509_V_OK)
+			BIO_printf(bio_err,"verify error:%s\n",
+				X509_verify_cert_error_string(verify_error));
+		else
+			ERR_print_errors(bio_err);
+		if (scon == NULL)
+			SSL_free(serverCon);
+		return NULL;
+		}
 
 	return serverCon;
 	}
