@@ -1,9 +1,9 @@
 /* apps/speed.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -61,6 +61,7 @@
 #undef SECONDS
 #define SECONDS		3	
 #define RSA_SECONDS	10	
+#define DSA_SECONDS	10	
 
 /* 11-Sep-92 Andrew Daviel   Support for Silicon Graphics IRIX added */
 /* 06-Apr-92 Luke Brennan    Support for VMS and add extra signal calls */
@@ -74,6 +75,7 @@
 #include <string.h>
 #include "apps.h"
 #include "crypto.h"
+#include "rand.h"
 #include "err.h"
 
 #ifndef MSDOS
@@ -106,22 +108,21 @@ struct tms {
 #include <sys/param.h>
 #endif
 
-#ifdef NO_DES
-#include <des.h>
-#else
+#ifndef NO_DES
 #include "des.h"
 #endif
-#ifdef NO_MD2
-#include <md2.h>
-#else
+#ifndef NO_MD2
 #include "md2.h"
 #endif
-#ifdef NO_MD5
-#include <md5.h>
-#else
+#ifndef NO_MDC2
+#include "mdc2.h"
+#endif
+#ifndef NO_MD5
 #include "md5.h"
 #endif
+#if !defined(NO_SHA) && !defined(NO_SHA1)
 #include "sha.h"
+#endif
 #ifndef NO_RC4
 #include "rc4.h"
 #endif
@@ -134,39 +135,48 @@ struct tms {
 #ifndef NO_BLOWFISH
 #include "blowfish.h"
 #endif
+#ifndef NO_RSA
 #include "rsa.h"
+#endif
 #include "x509.h"
-#include "./testcert.h"
+#include "./testrsa.h"
+#ifndef NO_DSA
+#include "./testdsa.h"
+#endif
 
 /* The following if from times(3) man page.  It may need to be changed */
 #ifndef HZ
-#ifndef CLK_TCK
-#ifndef VMS
-#define HZ	100.0
-#else /* VMS */
-#define HZ	100.0
-#endif
-#else /* CLK_TCK */
-#define HZ ((double)CLK_TCK)
-#endif
+# ifndef CLK_TCK
+#  ifndef _BSD_CLK_TCK_ /* FreeBSD hack */
+#   ifndef VMS
+#    define HZ	100.0
+#   else /* VMS */
+#    define HZ	100.0
+#   endif
+#  else /* _BSD_CLK_TCK_ */
+#   define HZ ((double)_BSD_CLK_TCK_)
+#  endif
+# else /* CLK_TCK */
+#  define HZ ((double)CLK_TCK)
+# endif
 #endif
 
 #undef BUFSIZE
 #define BUFSIZE	((long)1024*8)
-long run=0;
+int run=0;
 
 #ifndef NOPROTO
 static double Time_F(int s);
 static void print_message(char *s,long num,int length);
-static void rsa_print_message(long num,int bits);
+static void pkey_print_message(char *str,char *str2,long num,int bits,int sec);
 #else
 static double Time_F();
 static void print_message();
-static void rsa_print_message();
+static void pkey_print_message();
 #endif
 
 #ifdef SIGALRM
-#if defined(__STDC__) || defined(sgi)
+#if defined(__STDC__) || defined(sgi) || defined(_AIX)
 #define SIGRETTYPE void
 #else
 #define SIGRETTYPE int
@@ -208,7 +218,7 @@ int s;
 		{
 		times(&tend);
 		ret=((double)(tend.tms_utime-tstart.tms_utime))/HZ;
-		return((ret < 0.001)?0.001:ret);
+		return((ret < 1e-3)?1e-3:ret);
 		}
 #else /* !times() */
 	static struct timeb tstart,tend;
@@ -233,16 +243,27 @@ int MAIN(argc,argv)
 int argc;
 char **argv;
 	{
-	unsigned char *buf=NULL;
+	unsigned char *buf=NULL,*buf2=NULL;
 	int ret=1;
-#define ALGOR_NUM	10
+#define ALGOR_NUM	11
 #define SIZE_NUM	5
 #define RSA_NUM		4
-	long count;
-	int i,j,k;
+#define DSA_NUM		3
+	long count,rsa_count;
+	int i,j,k,rsa_num,rsa_num2;
+	unsigned int kk;
+#ifndef NO_MD2
 	unsigned char md2[MD2_DIGEST_LENGTH];
+#endif
+#ifndef NO_MDC2
+	unsigned char mdc2[MDC2_DIGEST_LENGTH];
+#endif
+#ifndef NO_MD5
 	unsigned char md5[MD5_DIGEST_LENGTH];
+#endif
+#if !defined(NO_SHA) || !defined(NO_SHA1)
 	unsigned char sha[SHA_DIGEST_LENGTH];
+#endif
 #ifndef NO_RC4
 	RC4_KEY rc4_ks;
 #endif
@@ -258,36 +279,46 @@ char **argv;
 	static unsigned char key16[16]=
 		{0x12,0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0,
 		 0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0,0x12};
-	des_cblock iv;
+	unsigned char iv[8];
+#ifndef NO_DES
 	static des_cblock key ={0x12,0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0};
 	static des_cblock key2={0x34,0x56,0x78,0x9a,0xbc,0xde,0xf0,0x12};
 	static des_cblock key3={0x56,0x78,0x9a,0xbc,0xde,0xf0,0x12,0x34};
 	des_key_schedule sch,sch2,sch3;
+#endif
 #define	D_MD2		0
-#define	D_MD5		1
-#define	D_SHA		2
-#define	D_SHA1		3
-#define	D_RC4		4
-#define	D_CBC_DES	5
-#define	D_EDE3_DES	6
-#define	D_CBC_IDEA	7
-#define	D_CBC_RC2	8
-#define	D_CBC_BF	9
+#define	D_MDC2		1
+#define	D_MD5		2
+#define	D_SHA		3
+#define	D_SHA1		4
+#define	D_RC4		5
+#define	D_CBC_DES	6
+#define	D_EDE3_DES	7
+#define	D_CBC_IDEA	8
+#define	D_CBC_RC2	9
+#define	D_CBC_BF	10
 	double d,results[ALGOR_NUM][SIZE_NUM];
 	static int lengths[SIZE_NUM]={8,64,256,1024,8*1024};
 	long c[ALGOR_NUM][SIZE_NUM];
 	static char *names[ALGOR_NUM]={
-		"md2","md5","sha","sha1","rc4",
+		"md2","mdc2","md5","sha","sha1","rc4",
 		"des cbc","des ede3","idea cbc",
 		"rc2 cbc","blowfish cbc"};
+#define	R_DSA_512	0
+#define	R_DSA_1024	1
+#define	R_DSA_2048	2
 #define	R_RSA_512	0
 #define	R_RSA_1024	1
 #define	R_RSA_2048	2
 #define	R_RSA_4096	3
 	RSA *rsa_key[RSA_NUM];
-	long rsa_c[RSA_NUM];
-	double rsa_results[RSA_NUM];
+	DSA *dsa_key[DSA_NUM];
+	long rsa_c[RSA_NUM][2];
+	long dsa_c[DSA_NUM][2];
+	double rsa_results[RSA_NUM][2];
+	double dsa_results[DSA_NUM][2];
 	static unsigned int rsa_bits[RSA_NUM]={512,1024,2048,4096};
+	static unsigned int dsa_bits[DSA_NUM]={512,1024,2048};
 	static unsigned char *rsa_data[RSA_NUM]=
 		{test512,test1024,test2048,test4096};
 	static int rsa_data_length[RSA_NUM]={
@@ -296,6 +327,7 @@ char **argv;
 	unsigned char *p;
 	int doit[ALGOR_NUM];
 	int rsa_doit[RSA_NUM];
+	int dsa_doit[DSA_NUM];
 	int pr_header=0;
 
 	apps_startup();
@@ -312,6 +344,11 @@ char **argv;
 		BIO_printf(bio_err,"out of memory\n");
 		goto end;
 		}
+	if ((buf2=(unsigned char *)Malloc((int)BUFSIZE)) == NULL)
+		{
+		BIO_printf(bio_err,"out of memory\n");
+		goto end;
+		}
 
 	memset(c,0,sizeof(c));
 	memset(iv,0,sizeof(iv));
@@ -320,43 +357,92 @@ char **argv;
 		doit[i]=0;
 	for (i=0; i<RSA_NUM; i++)
 		rsa_doit[i]=0;
+	for (i=0; i<DSA_NUM; i++)
+		dsa_doit[i]=0;
 	
 	j=0;
 	argc--;
 	argv++;
 	while (argc)
 		{
+#ifndef NO_MD2
 		if	(strcmp(*argv,"md2") == 0) doit[D_MD2]=1;
-		else if (strcmp(*argv,"md5") == 0) doit[D_MD5]=1;
-		else if (strcmp(*argv,"sha") == 0) doit[D_SHA]=1;
-		else if (strcmp(*argv,"sha1") == 0) doit[D_SHA1]=1;
-#ifndef NO_RC4
-		else if (strcmp(*argv,"rc4") == 0) doit[D_RC4]=1;
+		else
 #endif
-		else if (strcmp(*argv,"des-cbc") == 0) doit[D_CBC_DES]=1;
-		else if (strcmp(*argv,"des-ede3") == 0) doit[D_EDE3_DES]=1;
+#ifndef NO_MDC2
+			if (strcmp(*argv,"mdc2") == 0) doit[D_MDC2]=1;
+		else
+#endif
+#ifndef NO_MD5
+			if (strcmp(*argv,"md5") == 0) doit[D_MD5]=1;
+		else
+#endif
+#ifndef NO_SHA
+			if (strcmp(*argv,"sha") == 0) doit[D_SHA]=1;
+		else
+#endif
+#ifndef NO_SHA1
+			if (strcmp(*argv,"sha1") == 0) doit[D_SHA1]=1;
+		else
+#endif
+#ifndef NO_RC4
+			if (strcmp(*argv,"rc4") == 0) doit[D_RC4]=1;
+		else 
+#endif
+#ifndef NO_DEF
+			if (strcmp(*argv,"des-cbc") == 0) doit[D_CBC_DES]=1;
+		else	if (strcmp(*argv,"des-ede3") == 0) doit[D_EDE3_DES]=1;
+		else
+#endif
+#ifndef NO_RSA
+#ifdef RSAref
+			if (strcmp(*argv,"rsaref") == 0) 
+			{
+			RSA_set_default_method(RSA_PKCS1_RSAref());
+			j--;
+			}
+		else
+#endif
+			if (strcmp(*argv,"ssleay") == 0) 
+			{
+			RSA_set_default_method(RSA_PKCS1_SSLeay());
+			j--;
+			}
+		else
+#endif /* !NO_RSA */
+		     if (strcmp(*argv,"dsa512") == 0) dsa_doit[R_DSA_512]=2;
+		else if (strcmp(*argv,"dsa1024") == 0) dsa_doit[R_DSA_1024]=2;
+		else if (strcmp(*argv,"dsa2048") == 0) dsa_doit[R_DSA_2048]=2;
 		else if (strcmp(*argv,"rsa512") == 0) rsa_doit[R_RSA_512]=2;
 		else if (strcmp(*argv,"rsa1024") == 0) rsa_doit[R_RSA_1024]=2;
 		else if (strcmp(*argv,"rsa2048") == 0) rsa_doit[R_RSA_2048]=2;
 		else if (strcmp(*argv,"rsa4096") == 0) rsa_doit[R_RSA_4096]=2;
+		else
 #ifndef NO_RC2
-		else if (strcmp(*argv,"rc2-cbc") == 0) doit[D_CBC_RC2]=1;
+		     if (strcmp(*argv,"rc2-cbc") == 0) doit[D_CBC_RC2]=1;
 		else if (strcmp(*argv,"rc2") == 0) doit[D_CBC_RC2]=1;
+		else
 #endif
 #ifndef NO_IDEA
-		else if (strcmp(*argv,"idea-cbc") == 0) doit[D_CBC_IDEA]=1;
+		     if (strcmp(*argv,"idea-cbc") == 0) doit[D_CBC_IDEA]=1;
 		else if (strcmp(*argv,"idea") == 0) doit[D_CBC_IDEA]=1;
+		else
 #endif
 #ifndef NO_BLOWFISH
-		else if (strcmp(*argv,"bf-cbc") == 0) doit[D_CBC_BF]=1;
+		     if (strcmp(*argv,"bf-cbc") == 0) doit[D_CBC_BF]=1;
 		else if (strcmp(*argv,"blowfish") == 0) doit[D_CBC_BF]=1;
+		else
 #endif
-		else if (strcmp(*argv,"des") == 0)
+#ifndef NO_DES
+			if (strcmp(*argv,"des") == 0)
 			{
 			doit[D_CBC_DES]=1;
 			doit[D_EDE3_DES]=1;
 			}
-		else if (strcmp(*argv,"rsa") == 0)
+		else
+#endif
+#ifndef NO_RSA
+			if (strcmp(*argv,"rsa") == 0)
 			{
 			rsa_doit[R_RSA_512]=1;
 			rsa_doit[R_RSA_1024]=1;
@@ -364,9 +450,18 @@ char **argv;
 			rsa_doit[R_RSA_4096]=1;
 			}
 		else
+#endif
+#ifndef NO_DSA
+			if (strcmp(*argv,"dsa") == 0)
+			{
+			dsa_doit[R_DSA_512]=1;
+			dsa_doit[R_DSA_1024]=1;
+			}
+		else
+#endif
 			{
 			BIO_printf(bio_err,"bad value, pick one of\n");
-			BIO_printf(bio_err,"md2      md5      sha      sha1\n");
+			BIO_printf(bio_err,"md2      mdc2	md5      sha      sha1\n");
 #ifndef NO_IDEA
 			BIO_printf(bio_err,"idea-cbc ");
 #endif
@@ -398,6 +493,8 @@ char **argv;
 			doit[i]=1;
 		for (i=0; i<RSA_NUM; i++)
 			rsa_doit[i]=1;
+		for (i=0; i<DSA_NUM; i++)
+			dsa_doit[i]=1;
 		}
 	for (i=0; i<ALGOR_NUM; i++)
 		if (doit[i]) pr_header++;
@@ -407,6 +504,7 @@ char **argv;
 	BIO_printf(bio_err,"program when this computer is idle.\n");
 #endif
 
+#ifndef NO_RSA
 	for (i=0; i<RSA_NUM; i++)
 		{
 		p=rsa_data[i];
@@ -417,9 +515,19 @@ char **argv;
 			goto end;
 			}
 		}
+#endif
+
+#ifndef NO_DSA
+	dsa_key[0]=get_dsa512();
+	dsa_key[1]=get_dsa1024();
+	dsa_key[2]=get_dsa2048();
+#endif
+
+#ifndef NO_DES
 	des_set_key((C_Block *)key,sch);
 	des_set_key((C_Block *)key2,sch2);
 	des_set_key((C_Block *)key3,sch3);
+#endif
 #ifndef NO_IDEA
 	idea_set_encrypt_key(key16,&idea_ks);
 #endif
@@ -433,6 +541,7 @@ char **argv;
 	BF_set_key(&bf_ks,16,key16);
 #endif
 
+	memset(rsa_c,0,sizeof(rsa_c));
 #ifndef SIGALRM
 	BIO_printf(bio_err,"First we calculate the approximate speed ...\n");
 	count=10;
@@ -446,6 +555,7 @@ char **argv;
 		d=Time_F(STOP);
 		} while (d <3);
 	c[D_MD2][0]=count/10;
+	c[D_MDC2][0]=count/10;
 	c[D_MD5][0]=count;
 	c[D_SHA][0]=count;
 	c[D_SHA1][0]=count;
@@ -459,6 +569,7 @@ char **argv;
 	for (i=1; i<SIZE_NUM; i++)
 		{
 		c[D_MD2][i]=c[D_MD2][0]*4*lengths[0]/lengths[i];
+		c[D_MDC2][i]=c[D_MDC2][0]*4*lengths[0]/lengths[i];
 		c[D_MD5][i]=c[D_MD5][0]*4*lengths[0]/lengths[i];
 		c[D_SHA][i]=c[D_SHA][0]*4*lengths[0]/lengths[i];
 		c[D_SHA1][i]=c[D_SHA1][0]*4*lengths[0]/lengths[i];
@@ -476,15 +587,42 @@ char **argv;
 		c[D_CBC_RC2][i]=c[D_CBC_RC2][i-1]*l0/l1;
 		c[D_CBC_BF][i]=c[D_CBC_BF][i-1]*l0/l1;
 		}
-	rsa_c[R_RSA_512]=count/2000;
+	rsa_c[R_RSA_512][0]=count/2000;
+	rsa_c[R_RSA_512][1]=count/400;
 	for (i=1; i<RSA_NUM; i++)
 		{
-		rsa_c[i]=rsa_c[i-1]/8;
-		if ((rsa_doit[i] <= 1) && (rsa_c[i] == 0))
+		rsa_c[i][0]=rsa_c[i-1][0]/8;
+		rsa_c[i][1]=rsa_c[i-1][1]/4;
+		if ((rsa_doit[i] <= 1) && (rsa_c[i][0] == 0))
 			rsa_doit[i]=0;
 		else
-			{ if (rsa_c[i] == 0) rsa_c[i]=1; }
-		}				
+			{
+			if (rsa_c[i] == 0)
+				{
+				rsa_c[i][0]=1;
+				rsa_c[i][1]=20;
+				}
+			}				
+		}
+
+	dsa_c[R_DSA_512][0]=count/1000;
+	dsa_c[R_DSA_512][1]=count/1000/2;
+	for (i=1; i<DSA_NUM; i++)
+		{
+		dsa_c[i][0]=dsa_c[i-1][0]/4;
+		dsa_c[i][1]=dsa_c[i-1][1]/4;
+		if ((dsa_doit[i] <= 1) && (dsa_c[i][0] == 0))
+			dsa_doit[i]=0;
+		else
+			{
+			if (dsa_c[i] == 0)
+				{
+				dsa_c[i][0]=1;
+				dsa_c[i][1]=1;
+				}
+			}				
+		}
+
 #define COND(d)	(count != (d))
 #define COUNT(d) (d)
 #else
@@ -493,6 +631,7 @@ char **argv;
 	signal(SIGALRM,sig_done);
 #endif
 
+#ifndef NO_MD2
 	if (doit[D_MD2])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -507,7 +646,25 @@ char **argv;
 			results[D_MD2][j]=((double)count)/d*lengths[j];
 			}
 		}
+#endif
+#ifndef NO_MDC2
+	if (doit[D_MDC2])
+		{
+		for (j=0; j<SIZE_NUM; j++)
+			{
+			print_message(names[D_MDC2],c[D_MDC2][j],lengths[j]);
+			Time_F(START);
+			for (count=0,run=1; COND(c[D_MDC2][j]); count++)
+				MDC2(buf,(unsigned long)lengths[j],&(mdc2[0]));
+			d=Time_F(STOP);
+			BIO_printf(bio_err,"%ld %s's in %.2fs\n",
+				count,names[D_MDC2],d);
+			results[D_MDC2][j]=((double)count)/d*lengths[j];
+			}
+		}
+#endif
 
+#ifndef NO_MD5
 	if (doit[D_MD5])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -522,7 +679,9 @@ char **argv;
 			results[D_MD5][j]=((double)count)/d*lengths[j];
 			}
 		}
+#endif
 
+#ifndef NO_SHA
 	if (doit[D_SHA])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -537,7 +696,8 @@ char **argv;
 			results[D_SHA][j]=((double)count)/d*lengths[j];
 			}
 		}
-
+#endif
+#ifndef NO_SHA1
 	if (doit[D_SHA1])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -552,7 +712,7 @@ char **argv;
 			results[D_SHA1][j]=((double)count)/d*lengths[j];
 			}
 		}
-
+#endif
 #ifndef NO_RC4
 	if (doit[D_RC4])
 		{
@@ -570,7 +730,7 @@ char **argv;
 			}
 		}
 #endif
-
+#ifndef NO_DES
 	if (doit[D_CBC_DES])
 		{
 		for (j=0; j<SIZE_NUM; j++)
@@ -606,7 +766,7 @@ char **argv;
 			results[D_EDE3_DES][j]=((double)count)/d*lengths[j];
 			}
 		}
-
+#endif
 #ifndef NO_IDEA
 	if (doit[D_CBC_IDEA])
 		{
@@ -662,14 +822,19 @@ char **argv;
 		}
 #endif
 
+	RAND_bytes(buf,30);
+#ifndef NO_RSA
 	for (j=0; j<RSA_NUM; j++)
 		{
 		if (!rsa_doit[j]) continue;
-		rsa_print_message(rsa_c[j],rsa_bits[j]);
+		pkey_print_message("private","rsa",rsa_c[j][0],rsa_bits[j],
+			RSA_SECONDS);
 		Time_F(START);
-		for (count=0,run=1; COND(rsa_c[j]); count++)
+		for (count=0,run=1; COND(rsa_c[j][0]); count++)
 			{
-			if (RSA_private_encrypt(16,buf,buf,rsa_key[j]) < 0)
+			rsa_num=RSA_private_encrypt(30,buf,buf2,rsa_key[j],
+				RSA_PKCS1_PADDING);
+			if (rsa_num <= 0)
 				{
 				BIO_printf(bio_err,"RSA private encrypt failure\n");
 				ERR_print_errors(bio_err);
@@ -678,20 +843,116 @@ char **argv;
 				}
 			}
 		d=Time_F(STOP);
-		BIO_printf(bio_err,"%ld %d bit RSA's in %.2fs\n",
+		BIO_printf(bio_err,"%ld %d bit private RSA's in %.2fs\n",
 			count,rsa_bits[j],d);
-		rsa_results[j]=d/(double)count;
-		if (count <= 1)
+		rsa_results[j][0]=d/(double)count;
+		rsa_count=count;
+
+		pkey_print_message("public","rsa",rsa_c[j][1],rsa_bits[j],
+			RSA_SECONDS);
+		Time_F(START);
+		for (count=0,run=1; COND(rsa_c[j][1]); count++)
+			{
+			rsa_num2=RSA_public_decrypt(rsa_num,buf2,buf,rsa_key[j],
+				RSA_PKCS1_PADDING);
+			if (rsa_num2 <= 0)
+				{
+				BIO_printf(bio_err,"RSA public encrypt failure\n");
+				ERR_print_errors(bio_err);
+				count=1;
+				break;
+				}
+			}
+		d=Time_F(STOP);
+		BIO_printf(bio_err,"%ld %d bit public RSA's in %.2fs\n",
+			count,rsa_bits[j],d);
+		rsa_results[j][1]=d/(double)count;
+
+		if (rsa_count <= 1)
 			{
 			/* if longer than 10s, don't do any more */
 			for (j++; j<RSA_NUM; j++)
 				rsa_doit[j]=0;
 			}
 		}
+#endif
+
+	RAND_bytes(buf,20);
+#ifndef NO_DSA
+	for (j=0; j<DSA_NUM; j++)
+		{
+		if (!dsa_doit[j]) continue;
+		DSA_generate_key(dsa_key[j]);
+/*		DSA_sign_setup(dsa_key[j],NULL); */
+		pkey_print_message("sign","dsa",dsa_c[j][0],dsa_bits[j],
+			DSA_SECONDS);
+		Time_F(START);
+		for (count=0,run=1; COND(dsa_c[j][0]); count++)
+			{
+			rsa_num=DSA_sign(EVP_PKEY_DSA,buf,20,buf2,
+				&kk,dsa_key[j]);
+			if (rsa_num <= 0)
+				{
+				BIO_printf(bio_err,"DSA sign failure\n");
+				ERR_print_errors(bio_err);
+				count=1;
+				break;
+				}
+			}
+		d=Time_F(STOP);
+		BIO_printf(bio_err,"%ld %d bit DSA signs in %.2fs\n",
+			count,dsa_bits[j],d);
+		dsa_results[j][0]=d/(double)count;
+		rsa_count=count;
+
+		pkey_print_message("verify","dsa",dsa_c[j][1],dsa_bits[j],
+			DSA_SECONDS);
+		Time_F(START);
+		for (count=0,run=1; COND(dsa_c[j][1]); count++)
+			{
+			rsa_num2=DSA_verify(EVP_PKEY_DSA,buf,20,buf2,
+				kk,dsa_key[j]);
+			if (rsa_num2 <= 0)
+				{
+				BIO_printf(bio_err,"DSA verify failure\n");
+				ERR_print_errors(bio_err);
+				count=1;
+				break;
+				}
+			}
+		d=Time_F(STOP);
+		BIO_printf(bio_err,"%ld %d bit DSA verify in %.2fs\n",
+			count,dsa_bits[j],d);
+		dsa_results[j][1]=d/(double)count;
+
+		if (rsa_count <= 1)
+			{
+			/* if longer than 10s, don't do any more */
+			for (j++; j<DSA_NUM; j++)
+				dsa_doit[j]=0;
+			}
+		}
+#endif
 
 	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_VERSION));
         fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_BUILT_ON));
-	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_OPTIONS));
+	printf("options:");
+	printf("%s ",BN_options());
+#ifndef NO_MD2
+	printf("%s ",MD2_options());
+#endif
+#ifndef NO_RC4
+	printf("%s ",RC4_options());
+#endif
+#ifndef NO_DES
+	printf("%s ",des_options());
+#endif
+#ifndef NO_IDEA
+	printf("%s ",idea_options());
+#endif
+#ifndef NO_BLOWFISH
+	printf("%s ",BF_options());
+#endif
 	fprintf(stdout,"%s\n",SSLeay_version(SSLEAY_CFLAGS));
 
 	if (pr_header)
@@ -710,25 +971,48 @@ char **argv;
 		for (j=0; j<SIZE_NUM; j++)
 			{
 			if (results[k][j] > 10000)
-				fprintf(stdout," %11.2fk",results[k][j]/1000.0);
+				fprintf(stdout," %11.2fk",results[k][j]/1e3);
 			else
 				fprintf(stdout," %11.2f ",results[k][j]);
 			}
 		fprintf(stdout,"\n");
 		}
+#ifndef NO_RSA
+	j=1;
 	for (k=0; k<RSA_NUM; k++)
 		{
 		if (!rsa_doit[k]) continue;
-		fprintf(stdout,"rsa %4d bits %7.3fs",
-			rsa_bits[k],rsa_results[k]);
+		if (j) { printf("%18ssign    verify\n"," "); j=0; }
+		fprintf(stdout,"rsa %4d bits %8.4fs %8.4fs",
+			rsa_bits[k],rsa_results[k][0],rsa_results[k][1]);
 		fprintf(stdout,"\n");
 		}
+#endif
+#ifndef NO_DSA
+	j=1;
+	for (k=0; k<DSA_NUM; k++)
+		{
+		if (!dsa_doit[k]) continue;
+		if (j) { printf("%18ssign    verify\n"," "); j=0; }
+		fprintf(stdout,"dsa %4d bits %8.4fs %8.4fs",
+			dsa_bits[k],dsa_results[k][0],dsa_results[k][1]);
+		fprintf(stdout,"\n");
+		}
+#endif
 	ret=0;
 end:
 	if (buf != NULL) Free(buf);
+	if (buf2 != NULL) Free(buf2);
+#ifndef NO_RSA
 	for (i=0; i<RSA_NUM; i++)
 		if (rsa_key[i] != NULL)
 			RSA_free(rsa_key[i]);
+#endif
+#ifndef NO_DSA
+	for (i=0; i<DSA_NUM; i++)
+		if (dsa_key[i] != NULL)
+			DSA_free(dsa_key[i]);
+#endif
 	EXIT(ret);
 	}
 
@@ -750,16 +1034,19 @@ int length;
 #endif
 	}
 
-static void rsa_print_message(num,bits)
+static void pkey_print_message(str,str2,num,bits,tm)
+char *str;
+char *str2;
 long num;
 int bits;
+int tm;
 	{
 #ifdef SIGALRM
-	BIO_printf(bio_err,"Doing %d bit rsa's for %ds: ",bits,RSA_SECONDS);
+	BIO_printf(bio_err,"Doing %d bit %s %s's for %ds: ",bits,str,str2,tm);
 	BIO_flush(bio_err);
 	alarm(RSA_SECONDS);
 #else
-	BIO_printf(bio_err,"Doing %ld %d bit rsa's: ",num,bits);
+	BIO_printf(bio_err,"Doing %ld %d bit %s %s's: ",num,bits,str,str2);
 	BIO_flush(bio_err);
 #endif
 #ifdef LINT
