@@ -1,9 +1,9 @@
 /* crypto/asn1/asn1_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -59,6 +59,7 @@
 #include <stdio.h>
 #include "cryptlib.h"
 #include "asn1.h"
+#include "asn1_mac.h"
 
 #ifndef NOPROTO
 static int asn1_get_length(unsigned char **pp,int *inf,long *rl,int max);
@@ -68,7 +69,7 @@ static int asn1_get_length();
 static void asn1_put_length();
 #endif
 
-char *ASN1_version="ASN1 part of SSLeay 0.6.6b 29-Jun-1998";
+char *ASN1_version="ASN1 part of SSLeay 0.8.0 25-Jun-1997";
 
 int ASN1_check_infinite_end(p,len)
 unsigned char **p;
@@ -350,14 +351,20 @@ int len;
 	{
 	char *c;
 
-	if (str == NULL) return(0);
-	if (str->length <= len)
+	if (len < 0)
+		{
+		if (data == NULL)
+			return(0);
+		else
+			len=strlen((char *)data);
+		}
+	if ((str->length < len) || (str->data == NULL))
 		{
 		c=(char *)str->data;
 		if (c == NULL)
-			str->data=(unsigned char *)Malloc(len);
+			str->data=(unsigned char *)Malloc(len+1);
 		else
-			str->data=(unsigned char *)Realloc(c,len);
+			str->data=(unsigned char *)Realloc(c,len+1);
 
 		if (str->data == NULL)
 			{
@@ -366,7 +373,12 @@ int len;
 			}
 		}
 	str->length=len;
-	memcpy(str->data,data,len);
+	if (data != NULL)
+		{
+		memcpy(str->data,data,len);
+		/* an alowance for strings :-) */
+		str->data[len]='\0';
+		}
 	return(1);
 	}
 
@@ -418,3 +430,4 @@ ASN1_STRING *a,*b;
 	else
 		return(i);
 	}
+
