@@ -1,17 +1,22 @@
 /* crypto/asn1/asn1_par.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -52,19 +63,21 @@
 #include "x509.h"
 
 #ifndef NOPROTO
-static int asn1_print_info(BIO *bp, int tag, int xclass,int constructed);
+static int asn1_print_info(BIO *bp, int tag, int xclass,int constructed,
+	int indent);
 static int asn1_parse2(BIO *bp, unsigned char **pp, long length,
-	int offset, int depth);
+	int offset, int depth, int indent);
 #else
 static int asn1_print_info();
 static int asn1_parse2();
 #endif
 
-static int asn1_print_info(bp, tag, xclass, constructed)
+static int asn1_print_info(bp, tag, xclass, constructed,indent)
 BIO *bp;
 int tag;
 int xclass;
 int constructed;
+int indent;
 	{
 	static char *fmt="%-18s";
 	static char *fmt2="%2d %-15s";
@@ -74,7 +87,14 @@ int constructed;
 		p="cons: ";
 	else
 		p="prim: ";
-	if (BIO_puts(bp,p) <= 0) goto err;
+	if (BIO_write(bp,p,6) < 6) goto err;
+	if (indent)
+		{
+		if (indent > 128) indent=128;
+		memset(str,' ',indent);
+		if (BIO_write(bp,str,indent) < indent) goto err;
+		}
+
 	p=str;
 	if ((xclass & V_ASN1_PRIVATE) == V_ASN1_PRIVATE)
 		sprintf(str,"priv [ %d ] ",tag);
@@ -129,36 +149,41 @@ int constructed;
 		p2="(unknown)";
 		
 	if (p2 != NULL)
-		sprintf(str,fmt2,tag,p2);
+		{
+		if (BIO_printf(bp,fmt2,tag,p2) <= 0) goto err;
+		}
 	else
-		sprintf(str,fmt,p);
-	if (BIO_puts(bp,str) <= 0) goto err;
+		{
+		if (BIO_printf(bp,fmt,p) <= 0) goto err;
+		}
 	return(1);
 err:
 	return(0);
 	}
 
-int ASN1_parse(bp, pp, len)
+int ASN1_parse(bp, pp, len, indent)
 BIO *bp;
 unsigned char *pp;
 long len;
+int indent;
 	{
-	return(asn1_parse2(bp,&pp,len,0,0));
+	return(asn1_parse2(bp,&pp,len,0,0,indent));
 	}
 
-static int asn1_parse2(bp, pp, length, offset, depth)
+static int asn1_parse2(bp, pp, length, offset, depth, indent)
 BIO *bp;
 unsigned char **pp;
 long length;
 int offset;
 int depth;
+int indent;
 	{
-	char str[128];
 	unsigned char *p,*ep,*tot,*op,*opp;
 	long len;
 	int tag,xclass,ret=0;
 	int nl,hl,j,r;
 	ASN1_OBJECT *o=NULL;
+	ASN1_OCTET_STRING *os=NULL;
 
 	p= *pp;
 	tot=p+length;
@@ -170,37 +195,41 @@ int depth;
 #ifdef LINT
 		j=j;
 #endif
-		if (j == 0xff)
+		if (j & 0x80)
 			{
-			if (BIO_puts(bp,"Error in encoding\n") <= 0) goto end;
+			if (BIO_write(bp,"Error in encoding\n",18) <= 0)
+				goto end;
 			ret=0;
 			goto end;
 			}
 		hl=(p-op);
+		length-=hl;
 		/* if j == 0x21 it is a constructed indefinite length object */
-		sprintf(str,"%5ld:",(long)offset+(long)(op- *pp));
-		if (BIO_puts(bp,str) <= 0) goto end;
+		if (BIO_printf(bp,"%5ld:",(long)offset+(long)(op- *pp))
+			<= 0) goto end;
 
-		if (j != 0x21)
+		if (j != (V_ASN1_CONSTRUCTED | 1))
 			{
-			sprintf(str,"d=%-2d hl=%ld l=%4ld ",depth,(long)hl,len);
-			if (BIO_puts(bp,str) <= 0) goto end;
+			if (BIO_printf(bp,"d=%-2d hl=%ld l=%4ld ",
+				depth,(long)hl,len) <= 0)
+				goto end;
 			}
 		else
 			{
-			sprintf(str,"d=%-2d hl=%ld l=inf  ",depth,(long)hl);
-			if (BIO_puts(bp,str) <= 0) goto end;
+			if (BIO_printf(bp,"d=%-2d hl=%ld l=inf  ",
+				depth,(long)hl) <= 0)
+				goto end;
 			}
-		if (!asn1_print_info(bp,tag,xclass,j)) goto end;
+		if (!asn1_print_info(bp,tag,xclass,j,(indent)?depth:0))
+			goto end;
 		if (j & V_ASN1_CONSTRUCTED)
 			{
 			ep=p+len;
-			if (BIO_puts(bp,"\n") <= 0) goto end;
+			if (BIO_write(bp,"\n",1) <= 0) goto end;
 			if (len > length)
 				{
-				sprintf(str,"length is greater than %ld\n",
-					length);
-				if (BIO_puts(bp,str) <= 0) goto end;
+				BIO_printf(bp,
+					"length is greater than %ld\n",length);
 				ret=0;
 				goto end;
 				}
@@ -209,7 +238,8 @@ int depth;
 				for (;;)
 					{
 					r=asn1_parse2(bp,&p,(long)(tot-p),
-						offset+(p - *pp),depth+1);
+						offset+(p - *pp),depth+1,
+						indent);
 					if (r == 0) { ret=0; goto end; }
 					if ((r == 2) || (p >= tot)) break;
 					}
@@ -218,14 +248,15 @@ int depth;
 				while (p < ep)
 					{
 					r=asn1_parse2(bp,&p,(long)len,
-						offset+(p - *pp),depth+1);
+						offset+(p - *pp),depth+1,
+						indent);
 					if (r == 0) { ret=0; goto end; }
 					}
 			}
 		else if (xclass != 0)
 			{
 			p+=len;
-			if (BIO_puts(bp,"\n") <= 0) goto end;
+			if (BIO_write(bp,"\n",1) <= 0) goto end;
 			}
 		else
 			{
@@ -235,7 +266,7 @@ int depth;
 				(tag == V_ASN1_IA5STRING) ||
 				(tag == V_ASN1_UTCTIME))
 				{
-				if (BIO_puts(bp,":") <= 0) goto end;
+				if (BIO_write(bp,":",1) <= 0) goto end;
 				if (BIO_write(bp,(char *)p,(int)len)
 					!= (int)len)
 					goto end;
@@ -245,12 +276,12 @@ int depth;
 				opp=op;
 				if (d2i_ASN1_OBJECT(&o,&opp,len+hl) != NULL)
 					{
-					if (BIO_puts(bp,":") <= 0) goto end;
+					if (BIO_write(bp,":",1) <= 0) goto end;
 					i2a_ASN1_OBJECT(bp,o);
 					}
 				else
 					{
-					if (BIO_puts(bp,":BAD OBJECT") <= 0)
+					if (BIO_write(bp,":BAD OBJECT",11) <= 0)
 						goto end;
 					}
 				}
@@ -262,11 +293,43 @@ int depth;
 				ii=d2i_ASN1_BOOLEAN(NULL,&opp,len+hl);
 				if (ii < 0)
 					{
-					if (BIO_puts(bp,"Bad boolean\n"))
+					if (BIO_write(bp,"Bad boolean\n",12))
 						goto end;
 					}
-				sprintf(str,"%d",ii);
-				BIO_puts(bp,str);
+				BIO_printf(bp,":%d",ii);
+				}
+			else if (tag == V_ASN1_OCTET_STRING)
+				{
+				int i,printable=1;
+
+				opp=op;
+				os=d2i_ASN1_OCTET_STRING(NULL,&opp,len+hl);
+				if (os != NULL)
+					{
+					opp=os->data;
+					for (i=0; i<os->length; i++)
+						{
+						if ((	(opp[i] < ' ') &&
+							(opp[i] != '\n') &&
+							(opp[i] != '\r') &&
+							(opp[i] != '\t')) ||
+							(opp[i] > '~'))
+							{
+							printable=0;
+							break;
+							}
+						}
+					if (printable)
+						{
+						if (BIO_write(bp,":",1) <= 0)
+							goto end;
+						if (BIO_write(bp,(char *)opp,
+							os->length) <= 0)
+							goto end;
+						}
+					ASN1_OCTET_STRING_free(os);
+					os=NULL;
+					}
 				}
 			else if (tag == V_ASN1_INTEGER)
 				{
@@ -277,25 +340,25 @@ int depth;
 				bs=d2i_ASN1_INTEGER(NULL,&opp,len+hl);
 				if (bs != NULL)
 					{
-					if (BIO_puts(bp,":") <= 0) goto end;
+					if (BIO_write(bp,":",1) <= 0) goto end;
 					if (bs->type == V_ASN1_NEG_INTEGER)
-						if (BIO_puts(bp,"-") <= 0)
+						if (BIO_write(bp,"-",1) <= 0)
 							goto end;
 					for (i=0; i<bs->length; i++)
 						{
-						sprintf(str,"%02X",bs->data[i]);
-						if (BIO_puts(bp,str) <= 0)
+						if (BIO_printf(bp,"%02X",
+							bs->data[i]) <= 0)
 							goto end;
 						}
 					if (bs->length == 0)
 						{
-						if (BIO_puts(bp,"00") <= 0)
+						if (BIO_write(bp,"00",2) <= 0)
 							goto end;
 						}
 					}
 				else
 					{
-					if (BIO_puts(bp,"BAD INTEGER") <= 0)
+					if (BIO_write(bp,"BAD INTEGER",11) <= 0)
 						goto end;
 					}
 				ASN1_INTEGER_free(bs);
@@ -303,7 +366,7 @@ int depth;
 
 			if (!nl) 
 				{
-				if (BIO_puts(bp,"\n") <= 0) goto end;
+				if (BIO_write(bp,"\n",1) <= 0) goto end;
 				}
 			p+=len;
 			if ((tag == V_ASN1_EOC) && (xclass == 0))
@@ -312,10 +375,12 @@ int depth;
 				goto end;
 				}
 			}
+		length-=len;
 		}
 	ret=1;
 end:
 	if (o != NULL) ASN1_OBJECT_free(o);
+	if (os != NULL) ASN1_OCTET_STRING_free(os);
 	*pp=p;
 	return(ret);
 	}
