@@ -1,9 +1,9 @@
 /* crypto/bn/bn_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -58,13 +58,17 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "bn.h"
+#include "bn_lcl.h"
 
-char *BN_version="Big Number part of SSLeay 0.6.6b 29-Jun-1998";
+char *BN_version="Big Number part of SSLeay 0.8.0 25-Jun-1997";
 
-static BN_ULONG data_one=1L;
-static BIGNUM const_one={&data_one,1,1,0};
-BIGNUM *BN_value_one= &const_one;
+BIGNUM *BN_value_one()
+	{
+	static BN_ULONG data_one=1L;
+	static BIGNUM const_one={&data_one,1,1,0};
+
+	return(&const_one);
+	}
 
 char *BN_options()
 	{
@@ -107,12 +111,12 @@ BN_ULONG l;
 		8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
 		};
 
-#ifdef SIXTY_FOUR_BIT
-	if (l & 0xffffffff00000000)
+#ifdef SIXTY_FOUR_BIT_LONG
+	if (l & 0xffffffff00000000L)
 		{
-		if (l & 0xffff000000000000)
+		if (l & 0xffff000000000000L)
 			{
-			if (l & 0xff00000000000000)
+			if (l & 0xff00000000000000L)
 				{
 				return(bits[l>>56]+56);
 				}
@@ -120,7 +124,29 @@ BN_ULONG l;
 			}
 		else
 			{
-			if (l & 0x0000ff0000000000)
+			if (l & 0x0000ff0000000000L)
+				{
+				return(bits[l>>40]+40);
+				}
+			else	return(bits[l>>32]+32);
+			}
+		}
+	else
+#else
+#ifdef SIXTY_FOUR_BIT
+	if (l & 0xffffffff00000000LL)
+		{
+		if (l & 0xffff000000000000LL)
+			{
+			if (l & 0xff00000000000000LL)
+				{
+				return(bits[l>>56]+56);
+				}
+			else	return(bits[l>>48]+48);
+			}
+		else
+			{
+			if (l & 0x0000ff0000000000LL)
 				{
 				return(bits[l>>40]+40);
 				}
@@ -128,9 +154,10 @@ BN_ULONG l;
 			}
 		}
 	else
+#endif
 #endif
 		{
-#if defined(THIRTY_TWO_BIT) || defined(SIXTY_FOUR_BIT)
+#if defined(THIRTY_TWO_BIT) || defined(SIXTY_FOUR_BIT) || defined(SIXTY_FOUR_BIT_LONG)
 		if (l & 0xffff0000L)
 			{
 			if (l & 0xff000000L)
@@ -140,7 +167,7 @@ BN_ULONG l;
 		else
 #endif
 			{
-#if defined(SIXTEEN_BIT) || defined(THIRTY_TWO_BIT) || defined(SIXTY_FOUR_BIT)
+#if defined(SIXTEEN_BIT) || defined(THIRTY_TWO_BIT) || defined(SIXTY_FOUR_BIT) || defined(SIXTY_FOUR_BIT_LONG)
 			if (l & 0xff00L)
 				return(bits[l>>8]+8);
 			else	
@@ -227,6 +254,10 @@ BN_CTX *BN_CTX_new()
 		ret->bn[i]=n;
 		}
 
+	/* There is actually an extra one, this is for debugging my
+	 * stuff */
+	ret->bn[BN_CTX_NUM]=NULL;
+
 	ret->tos=0;
 	return(ret);
 err:
@@ -308,11 +339,17 @@ BIGNUM *a;
 
 	n=BN_num_bytes(a);
 	if (n > sizeof(unsigned long))
-		return(0xFFFFFFFF);
+#ifdef SIXTY_FOUR_BIT_LONG
+		return(BN_MASK2);
+#else
+		return(0xFFFFFFFFL);
+#endif
 	for (i=a->top-1; i>=0; i--)
 		{
+#ifndef SIXTY_FOUR_BIT /* the data item > unsigned long */
 		ret<<=BN_BITS4; /* stops the compiler complaining */
 		ret<<=BN_BITS4;
+#endif
 		ret|=a->d[i];
 		}
 	return(ret);
@@ -335,8 +372,10 @@ unsigned long w;
 		/* the following is done instead of
 		 * w>>=BN_BITS2 so compilers don't complain
 		 * on builds where sizeof(long) == BN_TYPES */
+#ifndef SIXTY_FOUR_BIT /* the data item > unsigned long */
 		w>>=BN_BITS4;
 		w>>=BN_BITS4;
+#endif
 		a->d[i]=(BN_ULONG)w&BN_MASK2;
 		if (a->d[i] != 0) a->top=i+1;
 		}
@@ -369,7 +408,7 @@ BIGNUM *ret;
 	ret->top=i;
 	while (n-- > 0)
 		{
-		l=(l<<8)| *(s++);
+		l=(l<<8L)| *(s++);
 		if (m-- == 0)
 			{
 			ret->d[--i]=l;
@@ -429,6 +468,15 @@ BIGNUM *b;
 	int gt,lt;
 	BN_ULONG t1,t2;
 
+	if ((a == NULL) || (b == NULL))
+		{
+		if (a != NULL)
+			return(-1);
+		else if (b != NULL)
+			return(1);
+		else
+			return(0);
+		}
 	if (a->neg != b->neg)
 		{
 		if (a->neg)
@@ -489,7 +537,7 @@ int n;
 	i=n/BN_BITS2;
 	j=n%BN_BITS2;
 	if (a->top <= i) return(0);
-	return((a->d[i]&(1L<<j))?1:0);
+	return((a->d[i]&(((BN_ULONG)1)<<j))?1:0);
 	}
 
 int BN_mask_bits(a,n)
