@@ -1,9 +1,9 @@
 /* crypto/conf/conf.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -95,7 +95,7 @@ static CONF_VALUE *get_section();
 
 #define scan_esc(p)	((*(++p) == '\0')?(p):(++p))
 
-char *CONF_version="CONF part of SSLeay 0.6.6b 29-Jun-1998";
+char *CONF_version="CONF part of SSLeay 0.8.0 25-Jun-1997";
 
 LHASH *CONF_load(h,file,line)
 LHASH *h;
@@ -105,10 +105,10 @@ long *line;
 	LHASH *ret=NULL;
 	FILE *in=NULL;
 #define BUFSIZE	512
-	int bufnum=0,i;
+	int bufnum=0,i,ii;
 	BUF_MEM *buff=NULL;
 	char *s,*p,*end;
-	int n,eline=0;
+	int again,n,eline=0;
 	CONF_VALUE *v=NULL,*vv,*tv;
 	CONF_VALUE *sv=NULL;
 	char *section=NULL,*buf;
@@ -121,10 +121,10 @@ long *line;
 		goto err;
 		}
 
-	in=fopen(file,"r");
+	in=fopen(file,"rb");
 	if (in == NULL)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		CONFerr(CONF_F_CONF_LOAD,ERR_R_SYS_LIB);
 		goto err;
 		}
@@ -151,7 +151,7 @@ long *line;
 	sv=new_section(ret,section);
 	if (sv == NULL)
 		{
-		CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREAT_NEW_SECTION);
+		CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREATE_NEW_SECTION);
 		goto err;
 		}
 	section_sk=(STACK *)sv->value;
@@ -159,6 +159,7 @@ long *line;
 	bufnum=0;
 	for (;;)
 		{
+		again=0;
 		if (!BUF_MEM_grow(buff,bufnum+BUFSIZE))
 			{
 			CONFerr(CONF_F_CONF_LOAD,ERR_R_BUF_LIB);
@@ -168,7 +169,7 @@ long *line;
 		*p='\0';
 		fgets(p,BUFSIZE-1,in);
 		p[BUFSIZE-1]='\0';
-		i=strlen(p);
+		ii=i=strlen(p);
 		if (i == 0) break;
 		while (i > 0)
 			{
@@ -177,31 +178,36 @@ long *line;
 			else
 				i--;
 			}
-		p[i++]='\n';
-		p[i]='\0';
+		/* we removed some trailing stuff so there is a new
+		 * line on the end. */
+		if (i == ii)
+			again=1; /* long line */
+		else
+			{
+			p[i]='\0';
+			eline++; /* another input line */
+			}
 
+		/* we now have a line with trailing \r\n removed */
+
+		/* i is the number of bytes */
 		bufnum+=i;
-		eline++;
 
 		v=NULL;
 		/* check for line continuation */
-		if (bufnum >= 2)
+		if (bufnum >= 1)
 			{
-			int n=0;
-
+			/* If we have bytes and the last char '\\' and
+			 * second last char is not '\\' */
 			p= &(buff->data[bufnum-1]);
-			while ((p != buff->data) && (*p == '\n'))
+			if (	IS_ESC(p[0]) &&
+				((bufnum <= 1) || !IS_ESC(p[-1])))
 				{
-				n++;
-				p--;
-				}
-			if (	((p >= buff->data) && IS_ESC(*p)) &&
-				((p == buff->data) || !IS_ESC(p[-1])))
-				{
-				bufnum-=n;
-				continue; 
+				bufnum--;
+				again=1;
 				}
 			}
+		if (again) continue;
 		bufnum=0;
 		buf=buff->data;
 
@@ -226,7 +232,7 @@ long *line;
 				sv=new_section(ret,section);
 			if (sv == NULL)
 				{
-				CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREAT_NEW_SECTION);
+				CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREATE_NEW_SECTION);
 				goto err;
 				}
 			section_sk=(STACK *)sv->value;
@@ -285,7 +291,7 @@ long *line;
 					tv=new_section(ret,psection);
 				if (tv == NULL)
 					{
-					CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREAT_NEW_SECTION);
+					CONFerr(CONF_F_CONF_LOAD,CONF_R_UNABLE_TO_CREATE_NEW_SECTION);
 					goto err;
 					}
 				ts=(STACK *)tv->value;
@@ -626,7 +632,7 @@ char *p;
 			p=scan_esc(p);
 			continue;
 			}
-		if (!IS_ALPHA_NUMERIC(*p))
+		if (!IS_ALPHA_NUMERIC_PUNCT(*p))
 			return(p);
 		p++;
 		}
