@@ -1,9 +1,9 @@
 /* crypto/cryptlib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -59,24 +59,6 @@
 #include <stdio.h>
 #include <string.h>
 #include "cryptlib.h"
-#include "bn.h"
-#ifdef NO_MD2
-#include <md2.h>
-#else
-#include "md2.h"
-#endif
-#include "rc4.h"
-#ifdef NO_DES
-#include <des.h>
-#else
-#include "des.h"
-#endif
-#ifndef NO_IDEA
-#include "idea.h"
-#endif
-#ifndef NO_BLOWFISH
-#include "blowfish.h"
-#endif
 #include "crypto.h"
 #include "date.h"
 
@@ -94,70 +76,16 @@ static char* lock_names[CRYPTO_NUM_LOCKS] =
 	"dsa",
 	"rsa",
 	"evp_pkey",
-	"certificate_ctx",
+	"x509_store",
 	"ssl_ctx",
 	"ssl_cert",
 	"ssl_session",
 	"ssl",
 	"rand",
 	"debug_malloc",
+	"BIO",
 	};
 
-char *SSLeay_version(t)
-int t;
-	{
-	if (t == SSLEAY_VERSION)
-		return("SSLeay 0.6.6b 29-Jun-1998");
-	if (t == SSLEAY_OPTIONS)
-		{
-		static char buf[100];
-
-		sprintf(buf,"options:%s ",BN_options());
-#ifndef NO_MD2
-		sprintf(&buf[strlen(buf)],"%s ",MD2_options());
-#endif
-		sprintf(&buf[strlen(buf)],"%s ",RC4_options());
-#ifndef NO_DES
-		sprintf(&buf[strlen(buf)],"%s ",des_options());
-#endif
-#ifndef NO_IDEA
-		sprintf(&buf[strlen(buf)],"%s ",idea_options());
-#endif
-#ifndef NO_BLOWFISH
-		sprintf(&buf[strlen(buf)],"%s",BF_options());
-#endif
-		return(buf);
-		}
-	if (t == SSLEAY_BUILT_ON)
-		{
-#ifdef DATE
-		static char buf[sizeof(DATE)+10];
-
-		sprintf(buf,"built on %s",DATE);
-        	return(buf);
-#else
-		return("build date not available");
-#endif
-		}
-	if (t == SSLEAY_CFLAGS)
-		{
-#ifdef CFLAGS
-		static char buf[sizeof(CFLAGS)+10];
-
-		sprintf(buf,"C flags:%s",CFLAGS);
-		return(buf);
-#else
-		return("C flags not available");
-#endif
-		}
-	return("not available");
-	}
-
-unsigned long SSLeay()
-	{
-	return(SSLEAY_VERSION_NUMBER);
-	}
-
 #ifndef NOPROTO
 static void (MS_FAR *locking_callback)(int mode,int type,
 	char *file,int line)=NULL;
