@@ -1,9 +1,9 @@
 /* crypto/dsa/dsa_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -64,7 +64,7 @@
 #include "dsa.h"
 #include "asn1.h"
 
-char *DSA_version="\0DSA part of SSLeay 0.6.6b 29-Jun-1998";
+char *DSA_version="\0DSA part of SSLeay 0.8.0 25-Jun-1997";
 
 DSA *DSA_new()
 	{
@@ -78,14 +78,17 @@ DSA *DSA_new()
 		}
 	ret->pad=0;
 	ret->version=0;
-	ret->x=NULL;
-	ret->y=NULL;
+	ret->write_params=1;
 	ret->p=NULL;
 	ret->q=NULL;
 	ret->g=NULL;
-/*	ret->dmp1=NULL;
-	ret->dmq1=NULL;
-	ret->iqmp=NULL;*/
+
+	ret->pub_key=NULL;
+	ret->priv_key=NULL;
+
+	ret->kinv=NULL;
+	ret->r=NULL;
+
 	ret->references=1;
 	return(ret);
 	}
@@ -107,14 +110,13 @@ DSA *r;
 		}
 #endif
 
-	if (r->x != NULL) BN_clear_free(r->x);
-	if (r->y != NULL) BN_clear_free(r->y);
 	if (r->p != NULL) BN_clear_free(r->p);
 	if (r->q != NULL) BN_clear_free(r->q);
 	if (r->g != NULL) BN_clear_free(r->g);
-/*	if (r->dmp1 != NULL) BN_clear_free(r->dmp1);
-	if (r->dmq1 != NULL) BN_clear_free(r->dmq1);
-	if (r->iqmp != NULL) BN_clear_free(r->iqmp);*/
+	if (r->pub_key != NULL) BN_clear_free(r->pub_key);
+	if (r->priv_key != NULL) BN_clear_free(r->priv_key);
+	if (r->kinv != NULL) BN_clear_free(r->kinv);
+	if (r->r != NULL) BN_clear_free(r->r);
 	Free(r);
 	}
 
@@ -122,13 +124,15 @@ int DSA_size(r)
 DSA *r;
 	{
 	int ret,i;
-	ASN1_BIT_STRING bs;
+	ASN1_INTEGER bs;
 	unsigned char buf[4];
 
-	bs.length=BN_num_bytes(r->q);
+	i=BN_num_bits(r->q);
+	bs.length=(i+7)/8;
 	bs.data=buf;
-	buf[0]=0xff;	
+	bs.type=V_ASN1_INTEGER;
 	/* If the top bit is set the asn1 encoding is 1 larger. */
+	buf[0]=0xff;	
 
 	i=i2d_ASN1_INTEGER(&bs,NULL);
 	i+=i; /* r and s */
