@@ -1,9 +1,9 @@
 /* crypto/evp/evp_enc.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -58,9 +58,9 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "envelope.h"
+#include "evp.h"
 
-char *EVP_version="EVP part of SSLeay 0.6.6b 29-Jun-1998";
+char *EVP_version="EVP part of SSLeay 0.8.0 25-Jun-1997";
 
 void EVP_CipherInit(ctx,data,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
@@ -108,7 +108,7 @@ unsigned char *iv;
 	{
 	if (cipher != NULL)
 		ctx->cipher=cipher;
-	ctx->cipher->enc_init(ctx,key,iv);
+	ctx->cipher->init(ctx,key,iv,1);
 	ctx->encrypt=1;
 	ctx->buf_len=0;
 	}
@@ -121,7 +121,7 @@ unsigned char *iv;
 	{
 	if (cipher != NULL)
 		ctx->cipher=cipher;
-	ctx->cipher->dec_init(ctx,key,iv);
+	ctx->cipher->init(ctx,key,iv,0);
 	ctx->encrypt=0;
 	ctx->buf_len=0;
 	}
@@ -286,3 +286,11 @@ int *outl;
 	return(1);
 	}
 
+void EVP_CIPHER_CTX_cleanup(c)
+EVP_CIPHER_CTX *c;
+	{
+	if ((c->cipher != NULL) && (c->cipher->cleanup != NULL))
+		c->cipher->cleanup(c);
+	memset(c,0,sizeof(EVP_CIPHER_CTX));
+	}
+
