@@ -1,9 +1,9 @@
 /* crypto/rsa/rsa_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -61,18 +61,46 @@
 #include "bn.h"
 #include "rsa.h"
 
-char *RSA_version="RSA part of SSLeay 0.6.6b 29-Jun-1998";
+char *RSA_version="RSA part of SSLeay 0.8.0 25-Jun-1997";
+
+static RSA_METHOD *default_RSA_meth=NULL;
 
 RSA *RSA_new()
+	{
+	return(RSA_new_method(NULL));
+	}
+
+void RSA_set_default_method(meth)
+RSA_METHOD *meth;
+	{
+	default_RSA_meth=meth;
+	}
+
+RSA *RSA_new_method(meth)
+RSA_METHOD *meth;
 	{
 	RSA *ret;
 
+	if (default_RSA_meth == NULL)
+		{
+#ifdef RSAref
+		default_RSA_meth=RSA_PKCS1_RSAref();
+#else
+		default_RSA_meth=RSA_PKCS1_SSLeay();
+#endif
+		}
 	ret=(RSA *)Malloc(sizeof(RSA));
 	if (ret == NULL)
 		{
-		RSAerr(RSA_F_RSA_NEW,ERR_R_MALLOC_FAILURE);
+		RSAerr(RSA_F_RSA_NEW_METHOD,ERR_R_MALLOC_FAILURE);
 		return(NULL);
 		}
+
+	if (meth == NULL)
+		ret->meth=default_RSA_meth;
+	else
+		ret->meth=meth;
+
 	ret->pad=0;
 	ret->version=0;
 	ret->n=NULL;
@@ -84,6 +112,12 @@ RSA *RSA_new()
 	ret->dmq1=NULL;
 	ret->iqmp=NULL;
 	ret->references=1;
+	ret->app_data=NULL;
+	if ((ret->meth->init != NULL) && !ret->meth->init(ret))
+		{
+		Free(ret);
+		ret=NULL;
+		}
 	return(ret);
 	}
 
@@ -104,6 +138,9 @@ RSA *r;
 		}
 #endif
 
+	if (r->meth->finish != NULL)
+		r->meth->finish(r);
+
 	if (r->n != NULL) BN_clear_free(r->n);
 	if (r->e != NULL) BN_clear_free(r->e);
 	if (r->d != NULL) BN_clear_free(r->d);
@@ -121,3 +158,43 @@ RSA *r;
 	return(BN_num_bytes(r->n));
 	}
 
+int RSA_public_encrypt(flen, from, to, rsa, padding)
+int flen;
+unsigned char *from;
+unsigned char *to;
+RSA *rsa;
+int padding;
+	{
+	return(rsa->meth->rsa_pub_enc(flen, from, to, rsa, padding));
+	}
+
+int RSA_private_encrypt(flen, from, to, rsa, padding)
+int flen;
+unsigned char *from;
+unsigned char *to;
+RSA *rsa;
+int padding;
+	{
+	return(rsa->meth->rsa_priv_enc(flen, from, to, rsa, padding));
+	}
+
+int RSA_private_decrypt(flen, from, to, rsa, padding)
+int flen;
+unsigned char *from;
+unsigned char *to;
+RSA *rsa;
+int padding;
+	{
+	return(rsa->meth->rsa_priv_dec(flen, from, to, rsa, padding));
+	}
+
+int RSA_public_decrypt(flen, from, to, rsa, padding)
+int flen;
+unsigned char *from;
+unsigned char *to;
+RSA *rsa;
+int padding;
+	{
+	return(rsa->meth->rsa_pub_dec(flen, from, to, rsa, padding));
+	}
+
