@@ -1,9 +1,9 @@
 /* crypto/txt_db/txt_db.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -66,7 +66,7 @@
 #undef BUFSIZE
 #define BUFSIZE	512
 
-char *TXT_DB_version="TXT_DB part of SSLeay 0.6.6b 29-Jun-1998";
+char *TXT_DB_version="TXT_DB part of SSLeay 0.8.0 25-Jun-1997";
 
 TXT_DB *TXT_DB_read(in,num)
 BIO *in;
@@ -191,20 +191,20 @@ err:
 		return(ret);
 	}
 
-char **TXT_DB_get_by_index(db,index,value)
+char **TXT_DB_get_by_index(db,idx,value)
 TXT_DB *db;
-int index;
+int idx;
 char **value;
 	{
 	char **ret;
 	LHASH *lh;
 
-	if (index >= db->num_fields)
+	if (idx >= db->num_fields)
 		{
 		db->error=DB_ERROR_INDEX_OUT_OF_RANGE;
 		return(NULL);
 		}
-	lh=db->index[index];
+	lh=db->index[idx];
 	if (lh == NULL)
 		{
 		db->error=DB_ERROR_NO_INDEX;
@@ -222,7 +222,7 @@ int (*qual)();
 unsigned long (*hash)();
 int (*cmp)();
 	{
-	LHASH *index;
+	LHASH *idx;
 	char *r;
 	int i,n;
 
@@ -231,7 +231,7 @@ int (*cmp)();
 		db->error=DB_ERROR_INDEX_OUT_OF_RANGE;
 		return(0);
 		}
-	if ((index=lh_new(hash,cmp)) == NULL)
+	if ((idx=lh_new(hash,cmp)) == NULL)
 		{
 		db->error=DB_ERROR_MALLOC;
 		return(0);
@@ -241,17 +241,17 @@ int (*cmp)();
 		{
 		r=(char *)sk_value(db->data,i);
 		if ((qual != NULL) && (qual(r) == 0)) continue;
-		if ((r=lh_insert(index,r)) != NULL)
+		if ((r=lh_insert(idx,r)) != NULL)
 			{
 			db->error=DB_ERROR_INDEX_CLASH;
 			db->arg1=sk_find(db->data,r);
 			db->arg2=i;
-			lh_free(index);
+			lh_free(idx);
 			return(0);
 			}
 		}
 	if (db->index[field] != NULL) lh_free(db->index[field]);
-	db->index[field]=index;
+	db->index[field]=idx;
 	db->qual[field]=qual;
 	return(1);
 	}
