@@ -1,17 +1,22 @@
 /* crypto/x509/x509_cmp.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,9 +60,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "cryptlib.h"
-#ifdef NO_MD5
-#include <md5.h>
-#else
+#ifndef NO_MD5
 #include "md5.h"
 #endif
 #include "objects.h"
@@ -66,25 +75,24 @@ X509 *b;
 
 	ai=a->cert_info;
 	bi=b->cert_info;
-	i=ASN1_BIT_STRING_cmp(ai->serialNumber,bi->serialNumber);
+	i=ASN1_INTEGER_cmp(ai->serialNumber,bi->serialNumber);
 	if (i) return(i);
-	return(X509_name_cmp(ai->issuer,bi->issuer));
+	return(X509_NAME_cmp(ai->issuer,bi->issuer));
 	}
 
+#ifndef NO_MD5
 unsigned long X509_issuer_and_serial_hash(a)
 X509 *a;
 	{
 	unsigned long ret=0;
 	MD5_CTX ctx;
 	unsigned char md[16];
-	char *str;
+	char str[256];
 
-	str=X509_NAME_oneline(a->cert_info->issuer);
-	if (str == NULL) return(0);
+	X509_NAME_oneline(a->cert_info->issuer,str,256);
 	ret=strlen(str);
 	MD5_Init(&ctx);
 	MD5_Update(&ctx,(unsigned char *)str,ret);
-	free(str);
 	MD5_Update(&ctx,(unsigned char *)a->cert_info->serialNumber->data,
 		(unsigned long)a->cert_info->serialNumber->length);
 	MD5_Final(&(md[0]),&ctx);
@@ -93,19 +101,27 @@ X509 *a;
 		)&0xffffffffL;
 	return(ret);
 	}
+#endif
 	
 int X509_issuer_name_cmp(a, b)
 X509 *a;
 X509 *b;
 	{
-	return(X509_name_cmp(a->cert_info->issuer,b->cert_info->issuer));
+	return(X509_NAME_cmp(a->cert_info->issuer,b->cert_info->issuer));
 	}
 
 int X509_subject_name_cmp(a, b)
 X509 *a;
 X509 *b;
 	{
-	return(X509_name_cmp(a->cert_info->subject,b->cert_info->subject));
+	return(X509_NAME_cmp(a->cert_info->subject,b->cert_info->subject));
+	}
+
+int X509_CRL_cmp(a, b)
+X509_CRL *a;
+X509_CRL *b;
+	{
+	return(X509_NAME_cmp(a->crl->issuer,b->crl->issuer));
 	}
 
 X509_NAME *X509_get_issuer_name(a)
@@ -117,7 +133,7 @@ X509 *a;
 unsigned long X509_issuer_name_hash(x)
 X509 *x;
 	{
-	return(X509_name_hash(x->cert_info->issuer));
+	return(X509_NAME_hash(x->cert_info->issuer));
 	}
 
 X509_NAME *X509_get_subject_name(a)
@@ -126,7 +142,7 @@ X509 *a;
 	return(a->cert_info->subject);
 	}
 
-ASN1_INTEGER *X509_get_serial_numberf(a)
+ASN1_INTEGER *X509_get_serialNumber(a)
 X509 *a;
 	{
 	return(a->cert_info->serialNumber);
@@ -135,22 +151,22 @@ X509 *a;
 unsigned long X509_subject_name_hash(x)
 X509 *x;
 	{
-	return(X509_name_hash(x->cert_info->subject));
+	return(X509_NAME_hash(x->cert_info->subject));
 	}
 
-int X509_name_cmp(a, b)
+int X509_NAME_cmp(a, b)
 X509_NAME *a;
 X509_NAME *b;
 	{
 	int i,j;
 	X509_NAME_ENTRY *na,*nb;
 
-	if (sk_num(a) != sk_num(b))
-		return(sk_num(a)-sk_num(b));
-	for (i=sk_num(a)-1; i>=0; i--)
+	if (sk_num(a->entries) != sk_num(b->entries))
+		return(sk_num(a->entries)-sk_num(b->entries));
+	for (i=sk_num(a->entries)-1; i>=0; i--)
 		{
-		na=(X509_NAME_ENTRY *)sk_value(a,i);
-		nb=(X509_NAME_ENTRY *)sk_value(b,i);
+		na=(X509_NAME_ENTRY *)sk_value(a->entries,i);
+		nb=(X509_NAME_ENTRY *)sk_value(b->entries,i);
 		j=na->value->length-nb->value->length;
 		if (j) return(j);
 		j=memcmp(na->value->data,nb->value->data,
@@ -163,31 +179,32 @@ X509_NAME *b;
 	/* We will check the object types after checking the values
 	 * since the values will more often be different than the object
 	 * types. */
-	for (i=sk_num(a)-1; i>=0; i--)
+	for (i=sk_num(a->entries)-1; i>=0; i--)
 		{
-		na=(X509_NAME_ENTRY *)sk_value(a,i);
-		nb=(X509_NAME_ENTRY *)sk_value(b,i);
+		na=(X509_NAME_ENTRY *)sk_value(a->entries,i);
+		nb=(X509_NAME_ENTRY *)sk_value(b->entries,i);
 		j=OBJ_cmp(na->object,nb->object);
 		if (j) return(j);
 		}
 	return(0);
 	}
 
+#ifndef NO_MD5
 /* I should do a DER encoding of the name and then hash it. */
-unsigned long X509_name_hash(x)
+unsigned long X509_NAME_hash(x)
 X509_NAME *x;
 	{
 	unsigned long ret=0;
 	unsigned char md[16];
-	char *str;
+	char str[256];
 
-	str=X509_NAME_oneline(x);
-	if (str == NULL) return(0);
+	X509_NAME_oneline(x,str,256);
 	ret=strlen(str);
 	MD5((unsigned char *)str,ret,&(md[0]));
-	free(str);
 	ret=(	((unsigned long)md[0]     )|((unsigned long)md[1]<<8L)|
 		((unsigned long)md[2]<<16L)|((unsigned long)md[3]<<24L)
 		)&0xffffffffL;
 	return(ret);
 	}
+#endif
+
