@@ -1,9 +1,9 @@
 /* e_os.h */
-/* Copyright (C) 1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -71,7 +71,6 @@ extern "C" {
 #define const
 #endif
 
-
 /********************************************************************
  The Microsoft section
  ********************************************************************/
@@ -102,6 +101,14 @@ extern "C" {
 
 #if defined(WINDOWS) || defined(MSDOS)
 
+#ifndef S_IFDIR
+#define S_IFDIR	_S_IFDIR
+#endif
+
+#ifndef S_IFMT
+#define S_IFMT	_S_IFMT
+#endif
+
 #define strncasecmp(a,b,c)	strnicmp((a),(b),(c))
 
 #  ifdef WINDOWS
@@ -120,13 +127,13 @@ extern "C" {
 #else
 #  define EXIT(n)		return(n);
 #endif
-#  define LIST_SEPERATOR_CHAR ';'
+#  define LIST_SEPARATOR_CHAR ';'
 #  define X_OK	0
 #  define W_OK	2
 #  define R_OK	4
 #  define SSLEAY_CONF	"ssleay.cnf"
 #  define NUL_DEV		"nul"
-#  define RFILE		"rand.dat"
+#  define RFILE		".rnd"
 
 #else /* The non-microsoft world world */
 
@@ -137,8 +144,8 @@ extern "C" {
 #  endif
 
 #  define SSLEAY_CONF	"ssleay.cnf"
-#  define RFILE		".rand"
-#  define LIST_SEPERATOR_CHAR ':'
+#  define RFILE		".rnd"
+#  define LIST_SEPARATOR_CHAR ':'
 #  ifndef MONOLITH
 #    define EXIT(n)		exit(n); return(n)
 #  else
@@ -156,17 +163,18 @@ extern "C" {
 #  if defined(WINDOWS) || defined(MSDOS)
       /* windows world */
 
-extern HINSTANCE _hInstance;
-
 #    ifdef NO_SOCK
 #      define SSLeay_Write(a,b,c)	(-1)
 #      define SSLeay_Read(a,b,c)	(-1)
 #      define SHUTDOWN(fd)		close(fd)
+#      define SHUTDOWN2(fd)		close(fd)
 #    else
 #      include <winsock.h>
+extern HINSTANCE _hInstance;
 #      define SSLeay_Write(a,b,c)	send((a),(b),(c),0)
 #      define SSLeay_Read(a,b,c)	recv((a),(b),(c),0)
 #      define SHUTDOWN(fd)		{ shutdown((fd),0); closesocket(fd); }
+#      define SHUTDOWN2(fd)		{ shutdown((fd),2); closesocket(fd); }
 #    endif
 
 
@@ -204,13 +212,16 @@ extern HINSTANCE _hInstance;
 #    define SSLeay_Read(a,b,c)     read((a),(b),(c))
 #    define SSLeay_Write(a,b,c)    write((a),(b),(c))
 #    define SHUTDOWN(fd)    { shutdown((fd),0); close((fd)); }
+#    define SHUTDOWN2(fd)   { shutdown((fd),2); close((fd)); }
 #    define INVALID_SOCKET	-1
 #  endif
 #endif
 
 #if defined(THREADS) || defined(sun)
+#ifndef _REENTRANT
 #define _REENTRANT
 #endif
+#endif
 
 /***********************************************/
 
