@@ -1,9 +1,9 @@
 /* ssl/s2_srvr.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -58,15 +58,13 @@
 
 #include <stdio.h>
 #ifdef NO_MD5
-#include <md5.h>
-#else
 #include "md5.h"
 #endif
-#include "buffer.h"
+#include "bio.h"
 #include "rand.h"
 #include "objects.h"
 #include "ssl_locl.h"
-#include "envelope.h"
+#include "evp.h"
 
 #ifndef NOPROTO
 static int get_client_master_key(SSL *s);
@@ -76,7 +74,8 @@ static int get_client_finished(SSL *s);
 static int server_verify(SSL *s);
 static int server_finish(SSL *s);
 static int request_certificate(SSL *s);
-static int ssl_cipher_pstrcmp(CIPHER **ap,CIPHER **bp);
+static int ssl_rsa_private_decrypt(CERT *c, int len, unsigned char *from,
+	unsigned char *to,int padding);
 #else
 static int get_client_master_key();
 static int get_client_hello();
@@ -85,19 +84,45 @@ static int get_client_finished();
 static int server_verify();
 static int server_finish();
 static int request_certificate();
-static int ssl_cipher_pstrcmp();
+static int ssl_rsa_private_decrypt();
 #endif
 
 #define BREAK	break
 
-int SSL2_accept(s)
+static SSL_METHOD *ssl2_get_server_method(ver)
+int ver;
+	{
+	if (ver == 2)
+		return(SSLv2_server_method());
+	else
+		return(NULL);
+	}
+
+SSL_METHOD *SSLv2_server_method()
+	{
+	static int init=1;
+	static SSL_METHOD SSLv2_server_data;
+
+	if (init)
+		{
+		init=0;
+		memcpy((char *)&SSLv2_server_data,(char *)sslv2_base_method(),
+			sizeof(SSL_METHOD));
+		SSLv2_server_data.ssl_accept=ssl2_accept;
+		SSLv2_server_data.get_ssl_method=ssl2_get_server_method;
+		}
+	return(&SSLv2_server_data);
+	}
+
+int ssl2_accept(s)
 SSL *s;
 	{
 	unsigned long l=time(NULL);
 	BUF_MEM *buf=NULL;
 	int ret= -1;
-	long num1,num2;
+	long num1;
 	void (*cb)()=NULL;
+	int new_state,state;
 
 	RAND_seed((unsigned char *)&l,sizeof(l));
 	ERR_clear_error();
@@ -109,7 +134,8 @@ SSL *s;
 		cb=s->ctx->info_callback;
 
 	/* init things to blank */
-	if (!SSL_in_init(s)) SSL_clear(s);
+	if (!SSL_in_init(s) || SSL_in_before(s)) SSL_clear(s);
+	s->in_handshake++;
 
 	if (((s->session == NULL) || (s->session->cert == NULL)) &&
 		(s->cert == NULL))
@@ -121,36 +147,35 @@ SSL *s;
 	errno=0;
 	for (;;)
 		{
-		int state;
-
-#ifdef SSL_DEBUG
-		SSL_TRACE(SSL_ERR,"state=%02X %d %s\n",
-			s->state,s->s2.init_num,SSL_state_string(s));
-#endif
 		state=s->state;
 
 		switch (s->state)
 			{
 		case SSL_ST_BEFORE:
 		case SSL_ST_ACCEPT:
-			if ((buf=BUF_MEM_new()) == NULL)
+		case SSL_ST_BEFORE|SSL_ST_ACCEPT:
+		case SSL_ST_OK|SSL_ST_ACCEPT:
+			s->version=2;
+			buf=s->init_buf;
+			if ((buf == NULL) && ((buf=BUF_MEM_new()) == NULL))
 				{ ret= -1; goto end; }
 			if (!BUF_MEM_grow(buf,(int)
-				SSL_MAX_RECORD_LENGTH_3_BYTE_HEADER))
+				SSL2_MAX_RECORD_LENGTH_3_BYTE_HEADER))
 				{ ret= -1; goto end; }
-			if (s->s2.init_buf) Free(s->s2.init_buf);
-			s->s2.init_buf=(unsigned char *)buf->data;
-			Free(buf);
-			s->s2.init_num=0;
-			s->state=SSL2_ST_GET_CLIENT_HELLO_A;
+			s->init_buf=buf;
+			s->init_num=0;
 			s->ctx->sess_accept++;
+			s->handshake_func=ssl2_accept;
+			s->state=SSL2_ST_GET_CLIENT_HELLO_A;
 			BREAK;
 
 		case SSL2_ST_GET_CLIENT_HELLO_A:
 		case SSL2_ST_GET_CLIENT_HELLO_B:
+		case SSL2_ST_GET_CLIENT_HELLO_C:
+			s->shutdown=0;
 			ret=get_client_hello(s);
 			if (ret <= 0) goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			s->state=SSL2_ST_SEND_SERVER_HELLO_A;
 			BREAK;
 
@@ -158,7 +183,7 @@ SSL *s;
 		case SSL2_ST_SEND_SERVER_HELLO_B:
 			ret=server_hello(s);
 			if (ret <= 0) goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			if (!s->hit)
 				{
 				s->state=SSL2_ST_GET_CLIENT_MASTER_KEY_A;
@@ -173,7 +198,7 @@ SSL *s;
 		case SSL2_ST_GET_CLIENT_MASTER_KEY_B:
 			ret=get_client_master_key(s);
 			if (ret <= 0) goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			s->state=SSL2_ST_SERVER_START_ENCRYPTION;
 			BREAK;
 
@@ -181,12 +206,9 @@ SSL *s;
 			/* Ok we how have sent all the stuff needed to
 			 * start encrypting, the next packet back will
 			 * be encrypted. */
-			if (!ssl_enc_init(s,0))
-				{
-				ret= -1;
-				goto end;
-				}
-			s->s2.clear_text=0;
+			if (!ssl2_enc_init(s,0))
+				{ ret= -1; goto end; }
+			s->s2->clear_text=0;
 			s->state=SSL2_ST_SEND_SERVER_VERIFY_A;
 			BREAK;
 
@@ -194,7 +216,7 @@ SSL *s;
 		case SSL2_ST_SEND_SERVER_VERIFY_B:
 			ret=server_verify(s);
 			if (ret <= 0) goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			if (s->hit)
 				{
 				/* If we are in here, we have been
@@ -215,16 +237,15 @@ SSL *s;
  			num1=BIO_ctrl(s->wbio,BIO_CTRL_INFO,0,NULL);
  			if (num1 != 0)
  				{
- 				num2=BIO_flush(s->wbio);
- 				if (num2 < 0) goto end;
- 				if (num1 != num2)
- 					{
- 					ret=0;
- 					goto end;
- 					}
- 				/* flushed and now remove buffering */
- 				s->wbio=BIO_pop(s->wbio);
- 				}
+				s->rwstate=SSL_WRITING;
+ 				num1=BIO_flush(s->wbio);
+ 				if (num1 <= 0) { ret= -1; goto end; }
+				s->rwstate=SSL_NOTHING;
+				}
+
+ 			/* flushed and now remove buffering */
+ 			s->wbio=BIO_pop(s->wbio);
+
  			s->state=SSL2_ST_GET_CLIENT_FINISHED_A;
   			BREAK;
 
@@ -233,7 +254,7 @@ SSL *s;
 			ret=get_client_finished(s);
 			if (ret <= 0)
 				goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			s->state=SSL2_ST_SEND_REQUEST_CERTIFICATE_A;
 			BREAK;
 
@@ -255,7 +276,7 @@ SSL *s;
 				{
 				ret=request_certificate(s);
 				if (ret <= 0) goto end;
-				s->s2.init_num=0;
+				s->init_num=0;
 				s->state=SSL2_ST_SEND_SERVER_FINISHED_A;
 				}
 			BREAK;
@@ -264,36 +285,20 @@ SSL *s;
 		case SSL2_ST_SEND_SERVER_FINISHED_B:
 			ret=server_finish(s);
 			if (ret <= 0) goto end;
-			s->s2.init_num=0;
+			s->init_num=0;
 			s->state=SSL_ST_OK;
 			break;
 
 		case SSL_ST_OK:
-			Free(s->s2.init_buf);
-			s->s2.init_buf=NULL;
-			s->s2.init_num=0;
+			BUF_MEM_free(s->init_buf);
+			s->init_buf=NULL;
+			s->init_num=0;
 		/*	ERR_clear_error();*/
 
-			/* If we want to cache the session-id, and we actually
-			 * get too, and there is a callback, then call it. */
-			if ((s->ctx->session_cache_mode & SSL_SESS_CACHE_SERVER)
-				&& !s->hit
-				&& SSL_add_session(s->ctx,s->session) &&
-				(s->ctx->new_session_cb != NULL))
-				{
-				CRYPTO_add(&s->session->references,1,
-					CRYPTO_LOCK_SSL_SESSION);
-				if (!s->ctx->new_session_cb(s,s->session))
-					SSL_SESSION_free(s->session);
-				}
-
-			/* flush cache every 255 entries */
-                        if (!((s->ctx->session_cache_mode &
-                                SSL_SESS_CACHE_NO_AUTO_CLEAR)) &&
-				((s->ctx->sess_accept_good & 0xff) == 0))
-                                SSL_flush_sessions(s->ctx,time(NULL));
+			ssl_update_cache(s,SSL_SESS_CACHE_SERVER);
 
 			s->ctx->sess_accept_good++;
+			/* s->server=1; */
 			ret=1;
 
 			goto end;
@@ -307,9 +312,15 @@ SSL *s;
 			}
 		
 		if ((cb != NULL) && (s->state != state))
+			{
+			new_state=s->state;
+			s->state=state;
 			cb(s,SSL_CB_ACCEPT_LOOP,1);
+			s->state=new_state;
+			}
 		}
 end:
+	s->in_handshake--;
 	if (cb != NULL)
 		cb(s,SSL_CB_ACCEPT_EXIT,ret);
 	return(ret);
@@ -318,22 +329,24 @@ end:
 static int get_client_master_key(s)
 SSL *s;
 	{
-	int i,j,n,keya;
+	int export,i,n,keya,error=0,ek;
 	unsigned char *p;
-	CIPHER *cp,c;
+	SSL_CIPHER *cp;
+	EVP_CIPHER *c;
+	EVP_MD *md;
 
-	p=s->s2.init_buf;
+	p=(unsigned char *)s->init_buf->data;
 	if (s->state == SSL2_ST_GET_CLIENT_MASTER_KEY_A)
 		{
-		i=SSL2_read(s,(char *)&(p[s->s2.init_num]),10-s->s2.init_num);
+		i=ssl2_read(s,(char *)&(p[s->init_num]),10-s->init_num);
 
-		if (i < (10-s->s2.init_num))
-			return(ssl_part_read(s,SSL_F_GET_CLIENT_MASTER_KEY,i));
-		if (*(p++) != SSL_MT_CLIENT_MASTER_KEY)
+		if (i < (10-s->init_num))
+			return(ssl2_part_read(s,SSL_F_GET_CLIENT_MASTER_KEY,i));
+		if (*(p++) != SSL2_MT_CLIENT_MASTER_KEY)
 			{
-			if (p[-1] != SSL_MT_ERROR)
+			if (p[-1] != SSL2_MT_ERROR)
 				{
-				ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+				ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 				SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_READ_WRONG_PACKET_TYPE);
 				}
 			else
@@ -342,89 +355,80 @@ SSL *s;
 			return(-1);
 			}
 
-		c.c1=p[0];
-		c.c2=p[1];
-		c.c3=p[2];
-		cp=(CIPHER *)OBJ_bsearch((char *)&c,(char *)ssl_ciphers,
-			SSL_NUM_CIPHERS,sizeof(CIPHER),(int (*)())ssl_cipher_cmp);
-		if ((cp == NULL) || (!cp->valid))
+		cp=ssl2_get_cipher_by_char(p);
+		if (cp == NULL)
 			{
-			ssl_return_error(s,SSL_PE_NO_CIPHER);
+			ssl2_return_error(s,SSL2_PE_NO_CIPHER);
 			SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,
 				SSL_R_NO_CIPHER_MATCH);
 			return(-1);
 			}
 		s->session->cipher= cp;
+
 		p+=3;
-		n2s(p,i); s->tmp.clear=i;
-		n2s(p,i); s->tmp.enc=i;
+		n2s(p,i); s->s2->tmp.clear=i;
+		n2s(p,i); s->s2->tmp.enc=i;
 		n2s(p,i); s->session->key_arg_length=i;
 		s->state=SSL2_ST_GET_CLIENT_MASTER_KEY_B;
-		s->s2.init_num=0;
+		s->init_num=0;
 		}
 
 	/* SSL2_ST_GET_CLIENT_MASTER_KEY_B */
-	p=s->s2.init_buf;
+	p=(unsigned char *)s->init_buf->data;
 	keya=s->session->key_arg_length;
-	n=s->tmp.clear+s->tmp.enc+keya - s->s2.init_num;
-	i=SSL2_read(s,(char *)&(p[s->s2.init_num]),n);
-	if (i != n) return(ssl_part_read(s,SSL_F_GET_CLIENT_MASTER_KEY,i));
+	n=s->s2->tmp.clear+s->s2->tmp.enc+keya - s->init_num;
+	i=ssl2_read(s,(char *)&(p[s->init_num]),n);
+	if (i != n) return(ssl2_part_read(s,SSL_F_GET_CLIENT_MASTER_KEY,i));
 
-	/* do key_arg before we unpack the crypted key. */
-	if (keya > 0)
-		{
-		s->session->key_arg=(unsigned char *)Malloc((unsigned int)keya);
-		if (s->session->key_arg == NULL)
-			{
-			ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
-			SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,
-				ERR_R_MALLOC_FAILURE);
-			return(-1);
-			}
-		}
-
-	memcpy(s->session->key_arg,&(p[s->tmp.clear+s->tmp.enc]),
+	memcpy(s->session->key_arg,&(p[s->s2->tmp.clear+s->s2->tmp.enc]),
 		(unsigned int)keya);
 
-	if (s->session->cert->privatekey == NULL)
+	if (s->session->cert->key->privatekey == NULL)
 		{
-		ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+		ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_NO_PRIVATEKEY);
 		return(-1);
 		}
-	i=s->cert->private_decrypt(s->cert,s->tmp.enc,
-		&(p[s->tmp.clear]),&(p[s->tmp.clear]));
-	/* j will be 0 or '5' for export ciphers */
-	j=s->session->cipher->enc_bits/8;
-	/* fix so we will not accept clear ciphers when using
-	 * non-export ciphers */
-	if (	(i < 0) ||
-		((j == 0) && (i != (int)(s->session->cipher->key_size))) ||
-		((j != 0) && ((i != j) || (s->tmp.clear+i !=
-			(unsigned int)(s->session->cipher->key_size)))))
+	i=ssl_rsa_private_decrypt(s->cert,s->s2->tmp.enc,
+		&(p[s->s2->tmp.clear]),&(p[s->s2->tmp.clear]),
+		(s->s2->ssl2_rollback)?RSA_SSLV23_PADDING:RSA_PKCS1_PADDING);
+
+	export=(s->session->cipher->algorithms & SSL_EXP)?1:0;
+	
+	if (!ssl_session_get_ciphers(s->session,&c,&md))
 		{
-#ifdef SSL_DEBUG
-		SSL_TRACE(SSL_ERR,"RSA decrypt error - i=%d enc_bits=%d\n",i,
-			s->session->cipher->enc_bits/8);
-#endif
-		/* If there is an rsa error, don't report it,
-		 * the MAC will fail later - eay 23-Jun-98
-		 */
-		if (j == 0)
-			i=s->session->cipher->key_size;
-		else
-			i=j;
-		/* Set the key to a dud value */
-		RAND_bytes(p,i);
-		/*
-		ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
-		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_BAD_RSA_DECRYPT);
-		return(-1);
-		*/
+		ssl2_return_error(s,SSL2_PE_NO_CIPHER);
+		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS);
+		return(0);
 		}
-	i+=s->tmp.clear;
+
+	if (export)
+		ek=(s->session->cipher->algorithm2 & SSL2_CF_8_BYTE_ENC)?8:5;
+	else
+		ek=5;
+
+	/* bad decrypt */
+	if (i < 0)
+		{
+		error=1;
+		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_BAD_RSA_DECRYPT);
+		}
+	/* incorrect number of key bytes for non export cipher */
+	else if ((!export && (i != EVP_CIPHER_key_length(c)))
+		|| ( export && ((i != ek) || (s->s2->tmp.clear+i !=
+			EVP_CIPHER_key_length(c)))))
+		{
+		error=1;
+		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_WRONG_NUMBER_OF_KEY_BITS);
+		}
+	if (error)
+		{
+		ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
+		return(-1);
+		}
+
+	if (export) i+=s->s2->tmp.clear;
 	s->session->master_key_length=i;
-	s->session->master_key=(unsigned char *)Malloc((unsigned int)i+1);
 	memcpy(s->session->master_key,p,(unsigned int)i);
 	return(1);
 	}
@@ -432,22 +436,32 @@ SSL *s;
 static int get_client_hello(s)
 SSL *s;
 	{
-	int i,j,n;
+	int i,n;
 	unsigned char *p;
-	CIPHER **cipher;
+	STACK *cs; /* a stack of SSL_CIPHERS */
+	STACK *cl; /* the ones we want to use */
+	int z;
 
-	p=s->s2.init_buf;
+	/* This is a bit of a hack to check for the correct packet
+	 * type the first time round. */
 	if (s->state == SSL2_ST_GET_CLIENT_HELLO_A)
 		{
-		i=SSL2_read(s,(char *)&(p[s->s2.init_num]),9-s->s2.init_num);
-		if (i < (9-s->s2.init_num)) 
-			return(ssl_part_read(s,SSL_F_GET_CLIENT_HELLO,i));
+		s->s2->first_packet=1;
+		s->state=SSL2_ST_GET_CLIENT_HELLO_B;
+		}
+
+	p=(unsigned char *)s->init_buf->data;
+	if (s->state == SSL2_ST_GET_CLIENT_HELLO_B)
+		{
+		i=ssl2_read(s,(char *)&(p[s->init_num]),9-s->init_num);
+		if (i < (9-s->init_num)) 
+			return(ssl2_part_read(s,SSL_F_GET_CLIENT_HELLO,i));
 	
-		if (*(p++) != SSL_MT_CLIENT_HELLO)
+		if (*(p++) != SSL2_MT_CLIENT_HELLO)
 			{
-			if (p[-1] != SSL_MT_ERROR)
+			if (p[-1] != SSL2_MT_ERROR)
 				{
-				ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+				ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 				SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_READ_WRONG_PACKET_TYPE);
 				}
 			else
@@ -456,70 +470,70 @@ SSL *s;
 			}
 		n2s(p,i);
 		if (i < s->version) s->version=i;
-		n2s(p,i); s->tmp.cipher_spec_length=i;
-		n2s(p,i); s->tmp.session_id_length=i;
-		n2s(p,i); s->challenge_length=i;
-		if (	(i < MIN_CHALLENGE_LENGTH) ||
-			(i > MAX_CHALLENGE_LENGTH))
+		n2s(p,i); s->s2->tmp.cipher_spec_length=i;
+		n2s(p,i); s->s2->tmp.session_id_length=i;
+		n2s(p,i); s->s2->challenge_length=i;
+		if (	(i < SSL2_MIN_CHALLENGE_LENGTH) ||
+			(i > SSL2_MAX_CHALLENGE_LENGTH))
 			{
 			SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_INVALID_CHALLENGE_LENGTH);
 			return(-1);
 			}
-		s->state=SSL2_ST_GET_CLIENT_HELLO_B;
-		s->s2.init_num=0;
+		s->state=SSL2_ST_GET_CLIENT_HELLO_C;
+		s->init_num=0;
 		}
 
-	/* SSL2_ST_GET_CLIENT_HELLO_B */
-	p=s->s2.init_buf;
-	n=s->tmp.cipher_spec_length+s->challenge_length+
-		s->tmp.session_id_length-s->s2.init_num;
-	i=SSL2_read(s,(char *)&(p[s->s2.init_num]),n);
-	if (i != n) return(ssl_part_read(s,SSL_F_GET_CLIENT_HELLO,i));
+	/* SSL2_ST_GET_CLIENT_HELLO_C */
+	p=(unsigned char *)s->init_buf->data;
+	n=s->s2->tmp.cipher_spec_length+s->s2->challenge_length+
+		s->s2->tmp.session_id_length-s->init_num;
+	i=ssl2_read(s,(char *)&(p[s->init_num]),n);
+	if (i != n) return(ssl2_part_read(s,SSL_F_GET_CLIENT_HELLO,i));
 
 	/* get session-id before cipher stuff so we can get out session
 	 * structure if it is cached */
 	/* session-id */
-	if ((s->tmp.session_id_length != 0) && 
-		(s->tmp.session_id_length != SSL_SESSION_ID_LENGTH))
+	if ((s->s2->tmp.session_id_length != 0) && 
+		(s->s2->tmp.session_id_length != SSL2_SSL_SESSION_ID_LENGTH))
 		{
-		ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+		ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 		SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_BAD_SSL_SESSION_ID_LENGTH);
 		return(-1);
 		}
 
-	if (s->tmp.session_id_length == 0)
+	if (s->s2->tmp.session_id_length == 0)
 		{
 		if (!ssl_get_new_session(s,1))
 			{
-			ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+			ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 			return(-1);
 			}
 		}
 	else
 		{
-		i=ssl_get_prev_session(s,s->tmp.session_id_length,
-			&(p[s->tmp.cipher_spec_length]));
+		i=ssl_get_prev_session(s,s->s2->tmp.session_id_length,
+			&(p[s->s2->tmp.cipher_spec_length]));
 		if (i == 1)
 			{ /* previous session */
 			s->hit=1;
 			}
 		else if (i == -1)
 			{
-			ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+			ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 			return(-1);
 			}
 		else
 			{
 			if (s->cert == NULL)
 				{
-				ssl_return_error(s,SSL_PE_NO_CERTIFICATE);
+				ssl2_return_error(s,SSL2_PE_NO_CERTIFICATE);
 				SSLerr(SSL_F_GET_CLIENT_HELLO,SSL_R_NO_CERTIFICATE_SET);
 				return(-1);
 				}
 
 			if (!ssl_get_new_session(s,1))
 				{
-				ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+				ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 				return(-1);
 				}
 			}
@@ -527,114 +541,53 @@ SSL *s;
 
 	if (!s->hit)
 		{
-		CIPHER cc,*cp,**cpp;
-		CIPHER *pref[SSL_NUM_CIPHERS],*good[SSL_NUM_CIPHERS];
-		int num_ciphers;
-		char *str;
+		cs=ssl_bytes_to_cipher_list(s,p,s->s2->tmp.cipher_spec_length,
+			&s->session->ciphers);
+		if (cs == NULL) goto mem_err;
 
-		cipher= &(s->session->ciphers[0]);
+		cl=ssl_get_ciphers_by_id(s);
 
-		/* First up, get a list of valid ciphers */
-		num_ciphers=0;
-		for (i=0; i<SSL_NUM_CIPHERS; i++)
+		for (z=0; z<sk_num(cs); z++)
 			{
-			if (ssl_ciphers[i].valid)
-				pref[num_ciphers++]= &(ssl_ciphers[i]);
-			good[i]=NULL;
-			}
-
-		/* sort by string name */
-		qsort((char *)&(pref[0]),num_ciphers,sizeof(CIPHER *),
-			(int (*)(P_CC_CC))ssl_cipher_pstrcmp);
-
-		/* now for each preference, copy across */
-		i=0;
-		cp= &cc;
-		for (;;)
-			{
-			str=SSL_get_cipher_list(s,i++);
-			if (str == NULL) break;
-			cc.name=str;
-			cpp=(CIPHER **)OBJ_bsearch((char *)&cp,
-				(char *)&(pref[0]),
-				num_ciphers,sizeof(CIPHER *),
-				(int (*)(P_CC_CC))ssl_cipher_pstrcmp);
-			if (cpp != NULL) good[cpp- &(pref[0])]= *cpp;
-			}
-		/* have a 'holy' list of ciphers, pack it down */
-		for (i=j=0; i<num_ciphers; i++)
-			if (good[i] != NULL) pref[j++]=good[i];
-
-		/* re-order it by cipher if */
-		num_ciphers=j;
-		qsort((char *)&(pref[0]),num_ciphers,sizeof(CIPHER *),
-			(int (*)(P_CC_CC))ssl_cipher_ptr_cmp);
-
-		/* pref now has 'num_ciphers' valid ciphers in it. */
-
-		cp= &cc;
-		for (j=i=0; i<(int)s->tmp.cipher_spec_length; i+=3)
-			{
-			cc.c1=p[i];
-			cc.c2=p[i+1];
-			cc.c3=p[i+2];
-
-			cpp=(CIPHER **)OBJ_bsearch((char *)&cp,
-				(char *)&(pref[0]),num_ciphers,sizeof(CIPHER *),
-				(int (*)())ssl_cipher_ptr_cmp);
-
-			/* If the cipher is valid, mark it as so */
-			if ((cpp != NULL) && (*cpp)->valid)
+			if (sk_find(cl,sk_value(cs,z)) < 0)
 				{
-				*(cipher++)= *cpp;
-				j++;
-				}
-			if (j == SSL_NUM_CIPHERS)
-				{
-				/* SOMETHING IS WRONG IF WE GET THIS IF THIS
-				 * IS NOT THE LAST ENTRY */
-				break;
+				sk_delete(cs,z);
+				z--;
 				}
 			}
 
-		/* Sort them by the 3 cipher numers, this is how we
-		 * look things up */
-		s->session->num_ciphers=j;
-		qsort((char *)&(s->session->ciphers[0]),j,sizeof(CIPHER *),
-			(int (*)(P_CC_CC))ssl_cipher_ptr_cmp);
-
+		/* s->session->ciphers should now have a list of
+		 * ciphers that are on both the client and server.
+		 * This list is ordered by the order the client sent
+		 * the ciphers.
+		 */
 		}
-	p+=s->tmp.cipher_spec_length;
+	p+=s->s2->tmp.cipher_spec_length;
 	/* done cipher selection */
 
 	/* session id extracted already */
-	p+=s->tmp.session_id_length;
+	p+=s->s2->tmp.session_id_length;
 
 	/* challenge */
-	if (s->challenge != NULL) Free(s->challenge);
-	s->challenge=(unsigned char *)Malloc((unsigned int)s->challenge_length);
-	if (s->challenge == NULL)
-		{
-		ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
-		SSLerr(SSL_F_GET_CLIENT_HELLO,ERR_R_MALLOC_FAILURE);
-		return(-1);
-		}
-	memcpy(s->challenge,p,(unsigned int)s->challenge_length);
+	memcpy(s->s2->challenge,p,(unsigned int)s->s2->challenge_length);
 	return(1);
+mem_err:
+	SSLerr(SSL_F_GET_CLIENT_HELLO,ERR_R_MALLOC_FAILURE);
+	return(0);
 	}
 
 static int server_hello(s)
 SSL *s;
 	{
 	unsigned char *p,*d;
-	int n,hit,i;
-	CIPHER **cipher;
+	int n,hit;
+	STACK *sk;
 
-	p=s->s2.init_buf;
+	p=(unsigned char *)s->init_buf->data;
 	if (s->state == SSL2_ST_SEND_SERVER_HELLO_A)
 		{
 		d=p+11;
-		*(p++)=SSL_MT_SERVER_HELLO;		/* type */
+		*(p++)=SSL2_MT_SERVER_HELLO;		/* type */
 		hit=s->hit;
 		*(p++)=(unsigned char)hit;
 		if (!hit)
@@ -659,7 +612,7 @@ SSL *s;
 
 		if (s->session->cert == NULL)
 			{
-			ssl_return_error(s,SSL_PE_NO_CERTIFICATE);
+			ssl2_return_error(s,SSL2_PE_NO_CERTIFICATE);
 			SSLerr(SSL_F_SERVER_HELLO,SSL_R_NO_CERTIFICATE_SPECIFIED);
 			return(-1);
 			}
@@ -673,55 +626,33 @@ SSL *s;
 			}
 		else
 			{
-			*(p++)=s->cert->cert_type; /* put certificate type */
+			/* EAY EAY */
+			/* put certificate type */
+			*(p++)=SSL2_CT_X509_CERTIFICATE;
 			s2n(s->version,p);	/* version */
-			n=i2d_X509(s->cert->x509,NULL);
+			n=i2d_X509(s->cert->key->x509,NULL);
 			s2n(n,p);		/* certificate length */
-			i2d_X509(s->cert->x509,&d);
+			i2d_X509(s->cert->key->x509,&d);
 			n=0;
 			
 			/* lets send out the ciphers we like in the
 			 * prefered order */
-			cipher= &(s->session->ciphers[0]);
-			for (i=0; i<s->session->num_ciphers; i++)
-				{
-				*(d++)=cipher[i]->c1;
-				*(d++)=cipher[i]->c2;
-				*(d++)=cipher[i]->c3;
-				n+=3;
-				}
+			sk= s->session->ciphers;
+			n=ssl_cipher_list_to_bytes(s,s->session->ciphers,d);
+			d+=n;
 			s2n(n,p);		/* add cipher length */
 			}
 
 		/* make and send conn_id */
-		s2n(CONECTION_ID_LENGTH,p);	/* add conn_id length */
-		if (s->conn_id == NULL)
-			s->conn_id=(unsigned char *)Malloc(CONECTION_ID_LENGTH);
-		if (s->conn_id == NULL)
-			{
-			ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
-			SSLerr(SSL_F_SERVER_HELLO,ERR_R_MALLOC_FAILURE);
-			return(-1);
-			}
-		s->conn_id_length=CONECTION_ID_LENGTH;
-		if (s->ctx->reverse)
-			{
-			if (s->ctx->conn_id_length != s->conn_id_length)
-				{
-				SSLerr(SSL_F_SERVER_HELLO,SSL_R_REVERSE_SSL_SESSION_ID_LENGTH_IS_WRONG);
-				return(-1);
-				}
-			memcpy(s->conn_id,s->ctx->conn_id,
-				(int)s->conn_id_length);
-			}
-		else
-			RAND_bytes(s->conn_id,(int)s->conn_id_length);
-		memcpy(d,s->conn_id,CONECTION_ID_LENGTH);
-		d+=CONECTION_ID_LENGTH;
+		s2n(SSL2_CONNECTION_ID_LENGTH,p);	/* add conn_id length */
+		s->s2->conn_id_length=SSL2_CONNECTION_ID_LENGTH;
+		RAND_bytes(s->s2->conn_id,(int)s->s2->conn_id_length);
+		memcpy(d,s->s2->conn_id,SSL2_CONNECTION_ID_LENGTH);
+		d+=SSL2_CONNECTION_ID_LENGTH;
 
 		s->state=SSL2_ST_SEND_SERVER_HELLO_B;
-		s->s2.init_num=d-s->s2.init_buf;
-		s->s2.init_off=0;
+		s->init_num=d-(unsigned char *)s->init_buf->data;
+		s->init_off=0;
 		}
 	/* SSL2_ST_SEND_SERVER_HELLO_B */
  	/* If we are using TCP/IP, the performace is bad if we do 2
@@ -749,7 +680,7 @@ SSL *s;
  		s->wbio=BIO_push(buf,s->wbio);
  		}
  
-	return(ssl_do_write(s));
+	return(ssl2_do_write(s));
 	}
 
 static int get_client_finished(s)
@@ -758,38 +689,38 @@ SSL *s;
 	unsigned char *p;
 	int i;
 
-	p=s->s2.init_buf;
+	p=(unsigned char *)s->init_buf->data;
 	if (s->state == SSL2_ST_GET_CLIENT_FINISHED_A)
 		{
-		i=SSL2_read(s,(char *)&(p[s->s2.init_num]),1-s->s2.init_num);
-		if (i < 1-s->s2.init_num)
-			return(ssl_part_read(s,SSL_F_GET_CLIENT_FINISHED,i));
+		i=ssl2_read(s,(char *)&(p[s->init_num]),1-s->init_num);
+		if (i < 1-s->init_num)
+			return(ssl2_part_read(s,SSL_F_GET_CLIENT_FINISHED,i));
 
-		if (*p != SSL_MT_CLIENT_FINISHED)
+		if (*p != SSL2_MT_CLIENT_FINISHED)
 			{
-			if (*p != SSL_MT_ERROR)
+			if (*p != SSL2_MT_ERROR)
 				{
-				ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+				ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 				SSLerr(SSL_F_GET_CLIENT_FINISHED,SSL_R_READ_WRONG_PACKET_TYPE);
 				}
 			else
 				SSLerr(SSL_F_GET_CLIENT_FINISHED,SSL_R_PEER_ERROR);
 			return(-1);
 			}
-		s->s2.init_num=0;
+		s->init_num=0;
 		s->state=SSL2_ST_GET_CLIENT_FINISHED_B;
 		}
 
 	/* SSL2_ST_GET_CLIENT_FINISHED_B */
-	i=SSL2_read(s,(char *)&(p[s->s2.init_num]),s->conn_id_length-s->s2.init_num);
-	if (i < (int)s->conn_id_length-s->s2.init_num)
+	i=ssl2_read(s,(char *)&(p[s->init_num]),s->s2->conn_id_length-s->init_num);
+	if (i < (int)s->s2->conn_id_length-s->init_num)
 		{
-		return(ssl_part_read(s,SSL_F_GET_CLIENT_FINISHED,i));
+		return(ssl2_part_read(s,SSL_F_GET_CLIENT_FINISHED,i));
 		}
-	if (memcmp(p,s->conn_id,(unsigned int)s->conn_id_length) != 0)
+	if (memcmp(p,s->s2->conn_id,(unsigned int)s->s2->conn_id_length) != 0)
 		{
-		ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
-		SSLerr(SSL_F_GET_CLIENT_FINISHED,SSL_R_CONECTION_ID_IS_DIFFERENT);
+		ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
+		SSLerr(SSL_F_GET_CLIENT_FINISHED,SSL_R_CONNECTION_ID_IS_DIFFERENT);
 		return(-1);
 		}
 	return(1);
@@ -802,16 +733,16 @@ SSL *s;
 
 	if (s->state == SSL2_ST_SEND_SERVER_VERIFY_A)
 		{
-		p=s->s2.init_buf;
-		*(p++)=SSL_MT_SERVER_VERIFY;
-		memcpy(p,s->challenge,(unsigned int)s->challenge_length);
-		/* p+=s->challenge_length; */
+		p=(unsigned char *)s->init_buf->data;
+		*(p++)=SSL2_MT_SERVER_VERIFY;
+		memcpy(p,s->s2->challenge,(unsigned int)s->s2->challenge_length);
+		/* p+=s->s2->challenge_length; */
 
 		s->state=SSL2_ST_SEND_SERVER_VERIFY_B;
-		s->s2.init_num=s->challenge_length+1;
-		s->s2.init_off=0;
+		s->init_num=s->s2->challenge_length+1;
+		s->init_off=0;
 		}
-	return(ssl_do_write(s));
+	return(ssl2_do_write(s));
 	}
 
 static int server_finish(s)
@@ -821,20 +752,20 @@ SSL *s;
 
 	if (s->state == SSL2_ST_SEND_SERVER_FINISHED_A)
 		{
-		p=s->s2.init_buf;
-		*(p++)=SSL_MT_SERVER_FINISHED;
+		p=(unsigned char *)s->init_buf->data;
+		*(p++)=SSL2_MT_SERVER_FINISHED;
 
 		memcpy(p,s->session->session_id,
 			(unsigned int)s->session->session_id_length);
 		/* p+=s->session->session_id_length; */
 
 		s->state=SSL2_ST_SEND_SERVER_FINISHED_B;
-		s->s2.init_num=s->session->session_id_length+1;
-		s->s2.init_off=0;
+		s->init_num=s->session->session_id_length+1;
+		s->init_off=0;
 		}
 
 	/* SSL2_ST_SEND_SERVER_FINISHED_B */
-	return(ssl_do_write(s));
+	return(ssl2_do_write(s));
 	}
 
 /* send the request and check the response */
@@ -845,108 +776,102 @@ SSL *s;
 	unsigned char *ccd;
 	int i,j,ctype,ret= -1;
 	X509 *x509=NULL;
+	STACK *sk=NULL;
 
-	if (s->tmp.ccl == NULL)
-		{
-		s->tmp.ccl=(unsigned char *)Malloc(MAX_CERT_CHALLENGE_LENGTH);
-		if (s->tmp.ccl == NULL)
-			{
-			SSLerr(SSL_F_REQUEST_CERTIFICATE,ERR_R_MALLOC_FAILURE);
-			goto msg_end;
-			}
-		}
-	ccd=s->tmp.ccl;
+	ccd=s->s2->tmp.ccl;
 	if (s->state == SSL2_ST_SEND_REQUEST_CERTIFICATE_A)
 		{
-		p=s->s2.init_buf;
-		*(p++)=SSL_MT_REQUEST_CERTIFICATE;
-		*(p++)=SSL_AT_MD5_WITH_RSA_ENCRYPTION;
-		RAND_bytes(ccd,MIN_CERT_CHALLENGE_LENGTH);
-		memcpy(p,ccd,MIN_CERT_CHALLENGE_LENGTH);
+		p=(unsigned char *)s->init_buf->data;
+		*(p++)=SSL2_MT_REQUEST_CERTIFICATE;
+		*(p++)=SSL2_AT_MD5_WITH_RSA_ENCRYPTION;
+		RAND_bytes(ccd,SSL2_MIN_CERT_CHALLENGE_LENGTH);
+		memcpy(p,ccd,SSL2_MIN_CERT_CHALLENGE_LENGTH);
 
 		s->state=SSL2_ST_SEND_REQUEST_CERTIFICATE_B;
-		s->s2.init_num=MIN_CERT_CHALLENGE_LENGTH+2;
-		s->s2.init_off=0;
+		s->init_num=SSL2_MIN_CERT_CHALLENGE_LENGTH+2;
+		s->init_off=0;
 		}
 
 	if (s->state == SSL2_ST_SEND_REQUEST_CERTIFICATE_B)
 		{
-		i=ssl_do_write(s);
+		i=ssl2_do_write(s);
 		if (i <= 0)
 			{
 			ret=i;
 			goto end;
 			}
 
-		s->s2.init_num=0;
+		s->init_num=0;
 		s->state=SSL2_ST_SEND_REQUEST_CERTIFICATE_C;
 		}
 
 	if (s->state == SSL2_ST_SEND_REQUEST_CERTIFICATE_C)
 		{
-		p=s->s2.init_buf;
-		i=SSL2_read(s,(char *)&(p[s->s2.init_num]),6-s->s2.init_num);
+		p=(unsigned char *)s->init_buf->data;
+		i=ssl2_read(s,(char *)&(p[s->init_num]),6-s->init_num);
 		if (i < 3)
 			{
-			ret=ssl_part_read(s,SSL_F_REQUEST_CERTIFICATE,i);
+			ret=ssl2_part_read(s,SSL_F_REQUEST_CERTIFICATE,i);
 			goto end;
 			}
 
-		if ((*p == SSL_MT_ERROR) && (i >= 3))
+		if ((*p == SSL2_MT_ERROR) && (i >= 3))
 			{
 			n2s(p,i);
 			if (s->verify_mode & SSL_VERIFY_FAIL_IF_NO_PEER_CERT)
 				{
-				ssl_return_error(s,SSL_PE_BAD_CERTIFICATE);
+				ssl2_return_error(s,SSL2_PE_BAD_CERTIFICATE);
 				SSLerr(SSL_F_REQUEST_CERTIFICATE,SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE);
 				goto end;
 				}
 			ret=1;
 			goto end;
 			}
-		if ((*(p++) != SSL_MT_CLIENT_CERTIFICATE) || (i < 6))
+		if ((*(p++) != SSL2_MT_CLIENT_CERTIFICATE) || (i < 6))
 			{
-			ssl_return_error(s,SSL_PE_UNDEFINED_ERROR);
+			ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 			SSLerr(SSL_F_REQUEST_CERTIFICATE,SSL_R_SHORT_READ);
 			goto end;
 			}
 		/* ok we have a response */
 		/* certificate type, there is only one right now. */
 		ctype= *(p++);
-		if (ctype != SSL_AT_MD5_WITH_RSA_ENCRYPTION)
+		if (ctype != SSL2_AT_MD5_WITH_RSA_ENCRYPTION)
 			{
-			ssl_return_error(s,SSL_PE_UNSUPPORTED_CERTIFICATE_TYPE);
+			ssl2_return_error(s,SSL2_PE_UNSUPPORTED_CERTIFICATE_TYPE);
 			SSLerr(SSL_F_REQUEST_CERTIFICATE,SSL_R_BAD_RESPONSE_ARGUMENT);
 			goto end;
 			}
-		n2s(p,i); s->tmp.clen=i;
-		n2s(p,i); s->tmp.rlen=i;
+		n2s(p,i); s->s2->tmp.clen=i;
+		n2s(p,i); s->s2->tmp.rlen=i;
 		s->state=SSL2_ST_SEND_REQUEST_CERTIFICATE_D;
-		s->s2.init_num=0;
+		s->init_num=0;
 		}
 
 	/* SSL2_ST_SEND_REQUEST_CERTIFICATE_D */
-	p=s->s2.init_buf;
-	j=s->tmp.clen+s->tmp.rlen-s->s2.init_num;
-	i=SSL2_read(s,(char *)&(p[s->s2.init_num]),j);
+	p=(unsigned char *)s->init_buf->data;
+	j=s->s2->tmp.clen+s->s2->tmp.rlen-s->init_num;
+	i=ssl2_read(s,(char *)&(p[s->init_num]),j);
 	if (i < j) 
 		{
-		ret=ssl_part_read(s,SSL_F_REQUEST_CERTIFICATE,i);
+		ret=ssl2_part_read(s,SSL_F_REQUEST_CERTIFICATE,i);
 		goto end;
 		}
 
-	x509=(X509 *)d2i_X509(NULL,&p,(long)s->tmp.clen);
+	x509=(X509 *)d2i_X509(NULL,&p,(long)s->s2->tmp.clen);
 	if (x509 == NULL)
 		{
 		SSLerr(SSL_F_REQUEST_CERTIFICATE,ERR_R_X509_LIB);
 		goto msg_end;
 		}
 
-        if (s->ctx->app_verify_callback != NULL)
-                i=s->ctx->app_verify_callback(s->ctx->app_verify_arg,s,x509);
-        else
-                i=X509_cert_verify(s->ctx->cert,x509,s->verify_callback,
-			&s->verify_result,(char *)s);
+	if (((sk=sk_new_null()) == NULL) || (!sk_push(sk,(char *)x509)))
+		{
+		SSLerr(SSL_F_REQUEST_CERTIFICATE,ERR_R_MALLOC_FAILURE);
+		goto msg_end;
+		}
+
+	i=ssl_verify_cert_chain(s,sk);
 
 	if (i)	/* we like the packet, now check the chksum */
 		{
@@ -954,11 +879,11 @@ SSL *s;
 		EVP_PKEY *pkey=NULL;
 
 		EVP_VerifyInit(&ctx,EVP_md5());
-		EVP_VerifyUpdate(&ctx,s->key_material,
-			(unsigned int)s->key_material_length);
-		EVP_VerifyUpdate(&ctx,ccd,MIN_CERT_CHALLENGE_LENGTH);
+		EVP_VerifyUpdate(&ctx,s->s2->key_material,
+			(unsigned int)s->s2->key_material_length);
+		EVP_VerifyUpdate(&ctx,ccd,SSL2_MIN_CERT_CHALLENGE_LENGTH);
 
-		i=i2d_X509(s->session->cert->x509,NULL);
+		i=i2d_X509(s->session->cert->key->x509,NULL);
 		buf2=(unsigned char *)Malloc((unsigned int)i);
 		if (buf2 == NULL)
 			{
@@ -966,15 +891,14 @@ SSL *s;
 			goto msg_end;
 			}
 		p2=buf2;
-		i=i2d_X509(s->session->cert->x509,&p2);
+		i=i2d_X509(s->session->cert->key->x509,&p2);
 		EVP_VerifyUpdate(&ctx,buf2,(unsigned int)i);
 		Free(buf2);
 
 		pkey=X509_get_pubkey(x509);
 		if (pkey == NULL) goto end;
-		i=EVP_VerifyFinal(&ctx,p,s->tmp.rlen,pkey);
+		i=EVP_VerifyFinal(&ctx,p,s->s2->tmp.rlen,pkey);
 		memset(&ctx,0,sizeof(ctx));
-		EVP_PKEY_free(pkey);
 
 		if (i) 
 			{
@@ -994,15 +918,40 @@ SSL *s;
 	else
 		{
 msg_end:
-		ssl_return_error(s,SSL_PE_BAD_CERTIFICATE);
+		ssl2_return_error(s,SSL2_PE_BAD_CERTIFICATE);
 		}
 end:
+	if (sk != NULL) sk_free(sk);
 	if (x509 != NULL) X509_free(x509);
 	return(ret);
 	}
 
-static int ssl_cipher_pstrcmp(ap,bp)
-CIPHER **ap,**bp;
+static int ssl_rsa_private_decrypt(c, len, from, to,padding)
+CERT *c;
+int len;
+unsigned char *from;
+unsigned char *to;
+int padding;
 	{
-	return(strcmp((*ap)->name,(*bp)->name));
+	RSA *rsa;
+	int i;
+
+	if ((c == NULL) || (c->key->privatekey == NULL))
+		{
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_NO_PRIVATEKEY);
+		return(-1);
+		}
+	if (c->key->privatekey->type != EVP_PKEY_RSA)
+		{
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_PUBLIC_KEY_IS_NOT_RSA);
+		return(-1);
+		}
+	rsa=c->key->privatekey->pkey.rsa;
+
+	/* we have the public key */
+	i=RSA_private_decrypt(len,from,to,rsa,padding);
+	if (i < 0)
+		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,ERR_R_RSA_LIB);
+	return(i);
 	}
+
