@@ -13,15 +13,18 @@ sub wanted
 	{
 	return unless /\.pl$/ || /^Configure$/;
 
-	open(IN,"<$dir/$_") || die "unable to open $dir/$_:$!\n";
+	open(IN,"<$_") || die "unable to open $dir/$_:$!\n";
 	@a=<IN>;
 	close(IN);
 
 	$a[0]="#!$ARGV[0]/perl\n";
 
-	# I'm not going to play safe, but then people should not be
-	# running this often.
-	open(OUT,">$dir/$_") || die "unable to open $dir/$_:$!\n";
+	# Playing it safe...
+	$new="$_.new";
+	open(OUT,">$new") || die "unable to open $dir/$new:$!\n";
 	print OUT @a;
 	close(OUT);
+
+	rename($new,$_) || die "unable to rename $dir/$new:$!\n";
+	chmod(0755,$_) || die "unable to chmod $dir/$new:$!\n";
 	}
