@@ -2,8 +2,8 @@
 # VCw16lib.pl - the file for Visual C++ 1.52b for windows, static libraries
 #
 
-$ssl=	"ssl16";
-$crypto="crypt16";
+$ssl=	"ssleay16";
+$crypto="libeay16";
 $RSAref="RSAref16";
 
 $o='\\';
@@ -76,14 +76,15 @@ $lfile='';
 
 $asm='ml /Cp /c /Cx';
 $afile='/Fo';
-if ($no_asm)
-	{
-	$bn_mulw_obj='';
-	$bn_mulw_src='';
-	$des_enc_obj='';
-	$des_enc_src='';
-	}
-else
+
+$bn_mulw_obj='';
+$bn_mulw_src='';
+$des_enc_obj='';
+$des_enc_src='';
+$bf_enc_obj='';
+$bf_enc_src='';
+
+if (!$no_asm)
 	{
 	if ($asmbits == 32)
 		{
@@ -95,25 +96,24 @@ else
 		$bn_mulw_obj='crypto\bn\asm\x86w16.obj';
 		$bn_mulw_src='crypto\bn\asm\x86w16.asm';
 		}
-	$des_enc_obj='';
-	$des_enc_src='';
 	}
 
 sub do_lib_rule
 	{
-	local($target,$name,$shlib)=@_;
+	local($objs,$target,$name,$shlib)=@_;
 	local($ret,$Name);
 
 	$taget =~ s/\//$o/g if $o ne '/';
 	($Name=$name) =~ tr/a-z/A-Z/;
 
-	$ret.="$target: \$(${Name}OBJ)\n";
+	$ret.="$target: $objs\n";
 #	$ret.="\t\$(RM) \$(O_$Name)\n";
 
 	# Due to a pathetic line length limit, I unwrap the args.
 	local($lib_names)="";
 	local($dll_names)="  \$(SHLIB_EX_OBJ) +\n";
-	foreach $_ (sort split(/\s+/,$Vars{"${Name}OBJ"}))
+	($obj)= ($objs =~ /\((.*)\)/);
+	foreach $_ (sort split(/\s+/,$Vars{$obj}))
 		{
 		$lib_names.="+$_ &\n";
 		$dll_names.="  $_ +\n";
@@ -126,11 +126,11 @@ sub do_lib_rule
 		}
 	else
 		{
-		local($ex)=($Name eq "SSL")?'$(L_CRYPTO)':"";
+		local($ex)=($target eq '$(O_SSL)')?'$(L_CRYPTO)':"";
 		$ex.=' winsock';
 		$ret.="\t\$(LINK) \$(MLFLAGS) @<<\n";
 		$ret.=$dll_names;
-		$ret.="\n  $target\n\n  $ex $libs\nms$o${name}16.def;\n<<\n";
+		$ret.="\n  $target\n\n  $ex $libs\nms$o${name}.def;\n<<\n";
 		($out_lib=$target) =~ s/O_/L_/;
 		$ret.="\timplib /noignorecase /nowep $out_lib $target\n";
 		}
